@@ -46,7 +46,7 @@
  * gcif_get_fw_info
  */
 int gcif_get_fw_info(mfile *mf,
-                     OUT struct connectib_icmd_get_fw_info* fw_info)
+                     OUT struct connectib_icmd_get_fw_info *fw_info)
 {
     SEND_ICMD_FLOW(mf, GET_FW_INFO, connectib_icmd_get_fw_info, fw_info, 0, 1);
 }
@@ -54,18 +54,24 @@ int gcif_get_fw_info(mfile *mf,
 /*
  * get_icmd_query_cap
  */
-int get_icmd_query_cap(mfile *mf, struct connectx4_icmd_query_cap_general* icmd_query_caps)
+int get_icmd_query_cap(mfile *mf, struct connectx4_icmd_query_cap_general *icmd_query_caps)
 {
     SEND_ICMD_FLOW(mf, GET_ICMD_QUERY_CAP, connectx4_icmd_query_cap_general, icmd_query_caps, 1, 0);
 }
 
-int gcif_mh_sync(mfile* mf, struct connectx4_icmd_mh_sync* mh_sync)
+int gcif_mh_sync(mfile *mf, struct connectx4_icmd_mh_sync *mh_sync)
 {
     SEND_ICMD_FLOW(mf, MH_SYNC_OPCODE, connectx4_icmd_mh_sync, mh_sync, 1, 0);
 }
 
-int gcif_mh_sync_status(mfile* mf, struct connectx4_icmd_mh_sync *mh_sync_out)
+int gcif_mh_sync_status(mfile *mf, struct connectx4_icmd_mh_sync *mh_sync_out)
 {
     memset(mh_sync_out, 0x0, sizeof(*mh_sync_out));
     SEND_ICMD_FLOW(mf, MH_SYNC_STATUS_OPCODE, connectx4_icmd_mh_sync, mh_sync_out, 1, 0);
 }
+
+int gcif_set_itrace(mfile *mf, struct connectib_itrace *itrace)
+{
+    SEND_ICMD_FLOW(mf, SET_ITRACE, connectib_itrace, itrace, 1, 0);
+}
+
