@@ -38,6 +38,7 @@
 
 
 #include <sstream>
+#include <iostream>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -75,16 +76,28 @@ using namespace std;
 #define OPERATION_NOT_SUPPORTED "Operation is not supported"
 
 #define CHECK_IF_FIELD_FOUND(b, s) \
-    if(!b) {\
-        throw MlxcfgException("The Parameter field %s was not initialized", s);\
+    if (!b) { \
+        throw MlxcfgException("The Parameter field %s was not initialized", s); \
     }
 
 
 #define CHECKFIELD(d, n) \
-    if(!d || (strcmp(d, "") == 0)) {\
-        throw MlxcfgException("The field: %s, cannot be empty", n);\
+    if (!d || (strcmp(d, "") == 0)) { \
+        throw MlxcfgException("The field: %s, cannot be empty", n); \
     }
 
+u_int32_t getSizeInBits(string size)
+{
+    u_int32_t bits = 0;
+    size_t pos = size.find('.');
+
+    if (pos != string::npos) {
+        bits += atoi(size.substr(pos + 1).c_str());
+    }
+    bits += (8 * strtol(size.substr(0, pos).c_str(), NULL, 0));
+    return bits;
+}
+
 Param::Param(int columnsCount, char **dataRow, char **headerRow) :
     _isNameFound(false), _isTLVNameFound(false), _isOffsetFound(false),
     _isSizeFound(false), _isTypeFound(false), _isDescriptionFound(false),
@@ -94,22 +107,25 @@ Param::Param(int columnsCount, char **dataRow, char **headerRow) :
     _isRegexFound(false), _isPortFound(false), _supportedFromVersion(0),
     _arrayLength(0)
 {
-
-    for(int i = 0; i < columnsCount; i++){
+    _offset = (Offset*)NULL;
+    for (int i = 0; i < columnsCount; i++) {
         if (strcmp(headerRow[i], PARAM_NAME_HEADER) == 0) {
             CHECKFIELD(dataRow[i], headerRow[i])
             _name = dataRow[i];
             _isNameFound = true;
         } else if (strcmp(headerRow[i], PARAM_TLVNAME_HEADER) == 0) {
             CHECKFIELD(dataRow[i], headerRow[i])
-            if(!dataRow[i]) {
+            if (!dataRow[i]) {
                 throw MlxcfgException("Empty tlv_name field");
             }
             _tlvName = dataRow[i];
             _isTLVNameFound = true;
         } else if (strcmp(headerRow[i], PARAM_OFFSET_HEADER) == 0) {
             CHECKFIELD(dataRow[i], headerRow[i])
-            _offset = dataRow[i];
+            if (_offset) {
+                delete _offset;
+            }
+            _offset = new Offset(dataRow[i]);
             _isOffsetFound = true;
         } else if (strcmp(headerRow[i], PARAM_SIZE_HEADER) == 0) {
             CHECKFIELD(dataRow[i], headerRow[i])
@@ -117,7 +133,7 @@ Param::Param(int columnsCount, char **dataRow, char **headerRow) :
             _isSizeFound = true;
         } else if (strcmp(headerRow[i], PARAM_TYPE_HEADER) == 0) {
             CHECKFIELD(dataRow[i], headerRow[i])
-           _type = Param::str2ParamType(dataRow[i]);
+            _type = Param::str2ParamType(dataRow[i]);
             _isTypeFound = true;
         } else if (strcmp(headerRow[i], PARAM_DESCRIPTION_HEADER) == 0) {
             //CHECKFIELD(dataRow[i], headerRow[i])
@@ -126,7 +142,7 @@ Param::Param(int columnsCount, char **dataRow, char **headerRow) :
         } else if (strcmp(headerRow[i], "version") == 0) {
             //TODO there is something in params regarding version.. check it
         } else if (strcmp(headerRow[i], PARAM_TEXTUAL_VALUES_HEADER) == 0) {
-            if(dataRow[i] != NULL) {
+            if (dataRow[i] != NULL) {
                 Param::str2TextualValuesMap(dataRow[i], _textualValues);
             }
             _isTextualValuesFound = true;
@@ -186,45 +202,51 @@ Param::Param(int columnsCount, char **dataRow, char **headerRow) :
 
     if (_type == ENUM && _textualValues.empty()) {
         throw MlxcfgException(
-                "The parameter: %s, "
-                "is type of ENUM but there is no any enum provided",
-                _name.c_str());
+                  "The parameter: %s, "
+                  "is type of ENUM but there is no any enum provided",
+                  _name.c_str());
     }
 
-    switch(_type) {
-        case BOOLEAN_TYPE:
-            _value = new BoolParamValue(_size);
-            break;
-        case ENUM:
-            if (_arrayLength == 0) {
-                _value = new EnumParamValue(_size, _textualValues);
-            } else {
-                _value = new EnumArrayParamValue(_size, _arrayLength, ENUM, _textualValues);
-            }
-           break;
-        case INTEGER:
-        case UNSIGNED:
-            if (_arrayLength == 0) {
-                _value = new UnsignedParamValue(_size);
-            } else {
-                _value = new ArrayParamValue(_size, _arrayLength, UNSIGNED);
-            }
-            break;
-        case BINARY:
-            if (_arrayLength == 0) {
-                _value = new BinaryParamValue(_size);
-            } else {
-                _value = new ArrayParamValue(_size, _arrayLength, BINARY);
-            }
-            break;
-        case STRING:
-            _value = new StringParamValue(_arrayLength);
-            break;
-        case BYTES:
-            _value = new BytesParamValue(_arrayLength);
-            break;
-        default:
-            throw MlxcfgException("Unknown parameter type %d", _type);
+    switch (_type) {
+    case BOOLEAN_TYPE:
+        _value = new BoolParamValue(_size);
+        break;
+
+    case ENUM:
+        if (_arrayLength == 0) {
+            _value = new EnumParamValue(_size, _textualValues);
+        } else {
+            _value = new EnumArrayParamValue(_size, _arrayLength, ENUM, _textualValues);
+        }
+        break;
+
+    case INTEGER:
+    case UNSIGNED:
+        if (_arrayLength == 0) {
+            _value = new UnsignedParamValue(_size);
+        } else {
+            _value = new ArrayParamValue(_size, _arrayLength, UNSIGNED);
+        }
+        break;
+
+    case BINARY:
+        if (_arrayLength == 0) {
+            _value = new BinaryParamValue(_size);
+        } else {
+            _value = new ArrayParamValue(_size, _arrayLength, BINARY);
+        }
+        break;
+
+    case STRING:
+        _value = new StringParamValue(_arrayLength);
+        break;
+
+    case BYTES:
+        _value = new BytesParamValue(_arrayLength);
+        break;
+
+    default:
+        throw MlxcfgException("Unknown parameter type %d", _type);
     }
 
 }
@@ -232,16 +254,17 @@ Param::Param(int columnsCount, char **dataRow, char **headerRow) :
 Param::~Param()
 {
     delete _value;
+    delete _offset;
 }
 
-void Param::pack(u_int8_t* buff)
+void Param::pack(u_int8_t *buff)
 {
-    _value->pack(buff, _offset);
+    _value->pack(buff, _offset->toBigEndian(getSizeInBits(_size), _arrayLength == 0));
 }
 
-void Param::unpack(u_int8_t* buff)
+void Param::unpack(u_int8_t *buff)
 {
-    _value->unpack(buff, _offset);
+    _value->unpack(buff, _offset->toBigEndian(getSizeInBits(_size), _arrayLength == 0));
 }
 
 void Param::setVal(string val)
@@ -250,7 +273,7 @@ void Param::setVal(string val)
         _value->setVal(val);
     } catch (MlxcfgException& e) {
         throw MlxcfgException("Cannot set the value %s for the parameter %s: %s",
-                        val.c_str(), getDisplayName().c_str(), e._err.c_str());
+                              val.c_str(), getDisplayName().c_str(), e._err.c_str());
     }
 }
 
@@ -282,8 +305,8 @@ string Param::getVal(u_int32_t index)
 
 void Param::getView(ParamView& paramView)
 {
-    if ((_type != BOOLEAN_TYPE && _type != ENUM && _type != INTEGER && _type != UNSIGNED && _type != BINARY)||
-        (_arrayLength !=0 && (_type == STRING || _type == BYTES))) {
+    if ((_type != BOOLEAN_TYPE && _type != ENUM && _type != INTEGER && _type != UNSIGNED && _type != BINARY) ||
+        (_arrayLength != 0 && (_type == STRING || _type == BYTES))) {
         throw MlxcfgException("Operation not supported");
     }
     paramView.mlxconfigName = _mlxconfigName;
@@ -291,7 +314,7 @@ void Param::getView(ParamView& paramView)
     paramView.type = _type;
     paramView.port = _port;
     paramView.textualVals = _textualValues;
-    if (_arrayLength !=0 && _type != STRING && _type != BYTES) {
+    if (_arrayLength != 0 && _type != STRING && _type != BYTES) {
         string strVal = "Array[0..";
         strVal += numToStr(_arrayLength - 1) + "]";
         paramView.strVal = strVal;
@@ -311,21 +334,21 @@ string Param::getDisplayName()
     return displayName;
 }
 
-ParamType Param::str2ParamType(const char* s)
+ParamType Param::str2ParamType(const char *s)
 {
-    if(strcmp(s, "BOOL") == 0) {
+    if (strcmp(s, "BOOL") == 0) {
         return BOOLEAN_TYPE;
-    } else if(strcmp(s, "ENUM") == 0) {
+    } else if (strcmp(s, "ENUM") == 0) {
         return ENUM;
-    } else if(strcmp(s, "INTEGER") == 0) {
+    } else if (strcmp(s, "INTEGER") == 0) {
         return INTEGER;
-    } else if(strcmp(s, "UNSIGNED") == 0) {
+    } else if (strcmp(s, "UNSIGNED") == 0) {
         return UNSIGNED;
-    } else if(strcmp(s, "STRING") == 0) {
+    } else if (strcmp(s, "STRING") == 0) {
         return STRING;
-    } else if(strcmp(s, "BINARY") == 0) {
+    } else if (strcmp(s, "BINARY") == 0) {
         return BINARY;
-    } else if(strcmp(s, "BYTES") == 0) {
+    } else if (strcmp(s, "BYTES") == 0) {
         return BYTES;
     }
     throw MlxcfgException("Unknown parameter type '%s'", s);
@@ -335,17 +358,17 @@ string Param::paramType2Str(enum ParamType paramType)
 {
     if (paramType == BOOLEAN_TYPE) {
         return "BOOL";;
-    } else if(paramType == ENUM) {
+    } else if (paramType == ENUM) {
         return "ENUM";
-    } else if(paramType == INTEGER) {
+    } else if (paramType == INTEGER) {
         return "INTEGER";
-    } else if(paramType == UNSIGNED) {
+    } else if (paramType == UNSIGNED) {
         return "UNSIGNED";
-    } else if(paramType == STRING) {
+    } else if (paramType == STRING) {
         return "STRING";
-    } else if(paramType == BINARY) {
+    } else if (paramType == BINARY) {
         return "BINARY";
-    } else if(paramType == BYTES) {
+    } else if (paramType == BYTES) {
         return "BYTES";
     } else {
         throw MlxcfgException("Unknown parameter type %d", paramType);
@@ -353,13 +376,13 @@ string Param::paramType2Str(enum ParamType paramType)
 }
 
 //split a string by comma, updates result vector with all sub-strings
-void splitByComma(const char* s, vector<string>& result)
+void splitByComma(const char *s, vector<string>& result)
 {
     int len = strlen(s);
     string e2v = "";
 
-    for(int i = 0; i < len; i++){
-        if (s[i] == ','){
+    for (int i = 0; i < len; i++) {
+        if (s[i] == ',') {
             result.push_back(e2v);
             e2v = "";
         } else {
@@ -383,16 +406,16 @@ void extractEnumAndValue(string e2v, string& e, u_int32_t& v)
 void Param::extractVars(vector<string>& result, string rule)
 {
     string tlv;
-    for(unsigned int i = 0; i < rule.size(); i++) {
-        if(rule[i] == '$') {
+    for (unsigned int i = 0; i < rule.size(); i++) {
+        if (rule[i] == '$') {
             tlv = "";
             i++;
             while (i < rule.size() &&
-                (rule[i] == '_' ||
-                 rule[i] == '.' ||
-                 ('a' <= rule[i] && rule[i] <= 'z') ||
-                 ('A' <= rule[i] && rule[i] <= 'Z') ||
-                 ('0' <= rule[i] && rule[i] <= '9'))) {
+                   (rule[i] == '_' ||
+                    rule[i] == '.' ||
+                    ('a' <= rule[i] && rule[i] <= 'z') ||
+                    ('A' <= rule[i] && rule[i] <= 'Z') ||
+                    ('0' <= rule[i] && rule[i] <= '9'))) {
                 tlv += rule[i];
                 i++;
             }
@@ -411,7 +434,7 @@ void Param::getRulesTLV(vector<string>& result)
     extractVars(vars, _minVal);
     extractVars(vars, _maxVal);
     extractVars(vars, _tempVars);
-    for(unsigned int i = 0; i < vars.size(); i++) {
+    for (unsigned int i = 0; i < vars.size(); i++) {
         size_t pos = vars[i].find('.');
         if (pos == string::npos) {
             continue;
@@ -421,7 +444,7 @@ void Param::getRulesTLV(vector<string>& result)
     }
 }
 
-void Param::str2TextualValuesMap(const char* s, map<string, u_int32_t>& m)
+void Param::str2TextualValuesMap(const char *s, map<string, u_int32_t>& m)
 {
     u_int32_t v;
     string e, e2v;
@@ -433,7 +456,7 @@ void Param::str2TextualValuesMap(const char* s, map<string, u_int32_t>& m)
     splitByComma(s, e2vVector);
 
     //extract the enum and update the map with this enum and its value
-    for(size_t i = 0; i < e2vVector.size(); i++){
+    for (size_t i = 0; i < e2vVector.size(); i++) {
         e2v = e2vVector[i];
         e = "";
         extractEnumAndValue(e2v, e, v);
@@ -445,10 +468,10 @@ void Param::genXMLTemplateAux(std::string& xmlTemplate, bool withVal, bool isPar
 {
     if (_type == ENUM) {
         xmlTemplate += "<!-- Legal Values:";
-        std::map<string,u_int32_t>::iterator tv = _textualValues.begin();
+        std::map<string, u_int32_t>::iterator tv = _textualValues.begin();
         xmlTemplate += tv->first;
         tv++;
-        for(; tv != _textualValues.end(); tv++) {
+        for (; tv != _textualValues.end(); tv++) {
             xmlTemplate += "|" + tv->first;
         }
         xmlTemplate += " -->\n";
@@ -456,8 +479,8 @@ void Param::genXMLTemplateAux(std::string& xmlTemplate, bool withVal, bool isPar
         xmlTemplate += "<!-- Legal Values: " DISABLED "/" ENABLED " -->\n";
     }
     xmlTemplate += "<" + _name + (isPartOfArray ? (" index='" + numToStr(index) + "' ")  : "") + ">" +
-            (withVal ? (isPartOfArray ? getVal(index) : getVal()) : "") +
-            "</" + _name + ">";
+                   (withVal ? (isPartOfArray ? getVal(index) : getVal()) : "") +
+                   "</" + _name + ">";
     xmlTemplate += isPartOfArray ? "\n" : "";
 }
 
@@ -498,21 +521,10 @@ void ParamValue::parseValue(string, u_int32_t&, string&)
     throw MlxcfgException(OPERATION_NOT_SUPPORTED);
 }
 
-u_int32_t ParamValue::getSizeInBits(string size)
-{
-    u_int32_t bits = 0;
-    size_t pos = size.find('.');
-
-    if (pos != string::npos) {
-        bits += atoi(size.substr(pos + 1).c_str());
-    }
-    bits += (8 * strtol(size.substr(0, pos).c_str(), NULL, 0));
-    return bits;
-}
-
 
 /* UnsignedParamValue Class */
-string UnsignedParamValue::getVal() {
+string UnsignedParamValue::getVal()
+{
     string strVal;
     stringstream ss;
 
@@ -534,40 +546,16 @@ void UnsignedParamValue::setVal(const u_int32_t value)
     _value = value;
 }
 
-void UnsignedParamValue::pack(u_int8_t* buff, u_int32_t bitOffset)
+void UnsignedParamValue::pack(u_int8_t *buff, u_int32_t bitOffset)
 {
     adb2c_push_bits_to_buff(buff, bitOffset, _size, _value);
 }
 
-void UnsignedParamValue::pack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0) + atoi(offset.substr(pos + 1).c_str());
-    bitOffset = 32 - bitOffset % 32 - _size + ((bitOffset >> 5) << 5);
-
-    pack(buff, bitOffset);
-}
-
-void UnsignedParamValue::unpack(u_int8_t* buff, u_int32_t bitOffset)
+void UnsignedParamValue::unpack(u_int8_t *buff, u_int32_t bitOffset)
 {
     _value = adb2c_pop_bits_from_buff(buff, bitOffset, _size);
 }
 
-void UnsignedParamValue::unpack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0) + atoi(offset.substr(pos + 1).c_str());
-    bitOffset = 32 - bitOffset % 32 - _size + ((bitOffset >> 5) << 5);
-
-    unpack(buff, bitOffset);
-}
-
 u_int32_t UnsignedParamValue::getIntVal()
 {
     return _value;
@@ -577,9 +565,9 @@ void UnsignedParamValue::parseValueAux(string strToParse, u_int32_t& value, stri
 {
     if (strToNum(strToParse, value, base)) {
         if ((_size == 32 && value == MLXCFG_UNKNOWN) ||
-                ((_size != 32) && value > (unsigned)((1 << _size) - 1))) {
+            ((_size != 32) && value > (unsigned)((1 << _size) - 1))) {
             throw MlxcfgException("The value %s is not valid,  as its size is %d bits",
-                    strToParse.c_str(), _size);
+                                  strToParse.c_str(), _size);
         }
     } else {
         throw MlxcfgException("Cannot parse the value");
@@ -594,13 +582,13 @@ void UnsignedParamValue::parseValue(string strToParse, u_int32_t& value, string&
 
 /* EnumParamValue Class */
 EnumParamValue::EnumParamValue(string size, map<string, u_int32_t> textualValues) :
-        UnsignedParamValue(size), _textualValues(textualValues)
+    UnsignedParamValue(size), _textualValues(textualValues)
 {
 
 }
 
 EnumParamValue::EnumParamValue(u_int32_t size, map<string, u_int32_t> textualValues) :
-        UnsignedParamValue(size), _textualValues(textualValues)
+    UnsignedParamValue(size), _textualValues(textualValues)
 {
 
 }
@@ -610,9 +598,9 @@ string EnumParamValue::getVal()
     map<string, u_int32_t>::const_iterator it;
 
     //printf("-D- textual values size = %u\n", (unsigned int)_textualValues.size());
-    for(it = _textualValues.begin(); it != _textualValues.end(); ++it) {
+    for (it = _textualValues.begin(); it != _textualValues.end(); ++it) {
         //printf("-D- textual value - %s=%d\n", it->first.c_str(), it->second);
-        if(it->second == _value) {
+        if (it->second == _value) {
             return it->first;
         }
     }
@@ -631,8 +619,8 @@ void EnumParamValue::setVal(u_int32_t value)
 {
     map<string, u_int32_t>::const_iterator it;
 
-    for(it = _textualValues.begin(); it != _textualValues.end(); ++it) {
-        if(it->second == value) {
+    for (it = _textualValues.begin(); it != _textualValues.end(); ++it) {
+        if (it->second == value) {
             _value = value;
             return;
         }
@@ -648,15 +636,15 @@ void EnumParamValue::parseValue(string strToParse, u_int32_t& value, string& str
     //try to convert to int
     if (strToNum(strToParse, value)) {
         //try to find the enum
-        for(it = _textualValues.begin(); it != _textualValues.end(); ++it) {
+        for (it = _textualValues.begin(); it != _textualValues.end(); ++it) {
             //printf("-D- textual value - %s=%d\n", it->first.c_str(), it->second);
-            if(it->second == value) {
+            if (it->second == value) {
                 strValue = it->first;
                 return;
             }
         }
     } else {
-        for(it = _textualValues.begin(); it != _textualValues.end(); ++it) {
+        for (it = _textualValues.begin(); it != _textualValues.end(); ++it) {
             if (strcasecmp(it->first.c_str(), strToParse.c_str()) == 0) {
                 value = it->second;
                 strValue = it->first;
@@ -671,7 +659,7 @@ void EnumParamValue::parseValue(string strToParse, u_int32_t& value, string& str
 
 /* BoolParamValue Class */
 BoolParamValue::BoolParamValue(string size) :
-        EnumParamValue(size, initBoolTextualValues())
+    EnumParamValue(size, initBoolTextualValues())
 {
 
 }
@@ -699,7 +687,7 @@ void BinaryParamValue::setVal(string val)
 
     if (val.size() > 8) {
         throw MlxcfgException("Value %s exceeds max allowed value(4 bytes)",
-                               tmpStr.c_str());
+                              tmpStr.c_str());
     }
     parseValue(val, _value, tmpStr);
 }
@@ -736,12 +724,12 @@ void StringParamValue::setVal(string val)
     _value = val;
 }
 
-void StringParamValue::pack(u_int8_t* buff, u_int32_t bitOffset)
+void StringParamValue::pack(u_int8_t *buff, u_int32_t bitOffset)
 {
     unsigned int i = 0;
     u_int32_t byteOffset = bitOffset / 8;
 
-    for(; i < _value.length(); i++) {
+    for (; i < _value.length(); i++) {
         buff[byteOffset + i] = _value[i];
     }
     if (_value.length() < _size) {
@@ -749,22 +737,11 @@ void StringParamValue::pack(u_int8_t* buff, u_int32_t bitOffset)
     }
 }
 
-void StringParamValue::pack(u_int8_t* buff, string offset)
-{
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    u_int32_t bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-
-    pack(buff, bitOffset);
-}
-
-void StringParamValue::unpack(u_int8_t* buff, u_int32_t bitOffset)
+void StringParamValue::unpack(u_int8_t *buff, u_int32_t bitOffset)
 {
     u_int32_t byteOffset = bitOffset / 8;
 
-    for(unsigned int i = 0; i < _size; i++) {
+    for (unsigned int i = 0; i < _size; i++) {
         if (buff[byteOffset + i] == '\0') {
             break;
         }
@@ -772,18 +749,6 @@ void StringParamValue::unpack(u_int8_t* buff, u_int32_t bitOffset)
     }
 }
 
-void StringParamValue::unpack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-
-    unpack(buff, bitOffset);
-}
-
 
 /* BytesParamValue Class */
 string BytesParamValue::getVal()
@@ -808,8 +773,8 @@ void BytesParamValue::setVal(string val)
 
     if (val.length() / 2 > _size) {
         throw MlxcfgException("Value size %d bytes is larger than max size: %d bytes",
-                val.length() / 2,
-                _size);
+                              val.length() / 2,
+                              _size);
     }
 
     u_int32_t i = 0;
@@ -828,7 +793,7 @@ void BytesParamValue::parseValue(string, u_int32_t&, string&)
     throw MlxcfgException("Not supported for BYTES types\n");
 }
 
-void BytesParamValue::pack(u_int8_t* buff, u_int32_t bitOffset)
+void BytesParamValue::pack(u_int8_t *buff, u_int32_t bitOffset)
 {
 
     VECTOR_ITERATOR(BinaryParamValue, _bytes, binary) {
@@ -837,19 +802,7 @@ void BytesParamValue::pack(u_int8_t* buff, u_int32_t bitOffset)
     }
 }
 
-void BytesParamValue::pack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-
-    pack(buff, bitOffset);
-}
-
-void BytesParamValue::unpack(u_int8_t* buff, u_int32_t bitOffset)
+void BytesParamValue::unpack(u_int8_t *buff, u_int32_t bitOffset)
 {
     VECTOR_ITERATOR(BinaryParamValue, _bytes, binary) {
         binary->unpack(buff, bitOffset);
@@ -857,18 +810,6 @@ void BytesParamValue::unpack(u_int8_t* buff, u_int32_t bitOffset)
     }
 }
 
-void BytesParamValue::unpack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-
-    unpack(buff, bitOffset);
-}
-
 void BytesParamValue::setVal(const vector<u_int32_t>& buffVal)
 {
     CONST_VECTOR_ITERATOR(u_int32_t, buffVal, it) {
@@ -881,28 +822,30 @@ void BytesParamValue::setVal(const vector<u_int32_t>& buffVal)
 
 
 ArrayParamValue::ArrayParamValue(string size, u_int32_t count, enum ParamType paramType) :
-        ParamValue(size), _elementSizeInBits(_size / count)
+    ParamValue(size), _elementSizeInBits(_size / count)
 {
     if (_elementSizeInBits != 32 &&
-            _elementSizeInBits != 16 &&
-            _elementSizeInBits != 8) {
-        throw MlxcfgException("The size of the elements of array values must be 1, 2 or 4 bytes!");
+        _elementSizeInBits != 16 &&
+        _elementSizeInBits != 8) {
+        throw MlxcfgException("The size of the elements of array values must be 1, 2 or 4 bytes.");
     }
     if (paramType == ENUM) {
         return;
     }
-    for(u_int32_t i = 0; i < count; i++) {
-        ParamValue* t;
+    for (u_int32_t i = 0; i < count; i++) {
+        ParamValue *t;
         switch (paramType) {
-            case UNSIGNED:
-                t = new UnsignedParamValue(_elementSizeInBits);
-                break;
-            case BINARY:
-                t = new BinaryParamValue(_elementSizeInBits);
-                break;
-            default:
-                throw MlxcfgException("% is not supported in Array parameters!", Param::paramType2Str(paramType).c_str());
-                break;
+        case UNSIGNED:
+            t = new UnsignedParamValue(_elementSizeInBits);
+            break;
+
+        case BINARY:
+            t = new BinaryParamValue(_elementSizeInBits);
+            break;
+
+        default:
+            throw MlxcfgException("% is not supported in Array parameters!", Param::paramType2Str(paramType).c_str());
+            break;
         }
         _values.push_back(t);
     }
@@ -910,7 +853,7 @@ ArrayParamValue::ArrayParamValue(string size, u_int32_t count, enum ParamType pa
 
 ArrayParamValue::~ArrayParamValue()
 {
-    for(u_int32_t i = 0; i < _values.size(); i++) {
+    for (u_int32_t i = 0; i < _values.size(); i++) {
         if (_values[i] != NULL) {
             delete _values[i];
         }
@@ -945,8 +888,8 @@ void ArrayParamValue::setVal(vector<string> vals)
 {
     if (_values.size() != vals.size()) {
         throw MlxcfgException(
-                "The size of the given input %d is not the same as the size of the parameter array %d",
-                vals.size(), _values.size());
+                  "The size of the given input %d is not the same as the size of the parameter array %d",
+                  vals.size(), _values.size());
     }
     for (u_int32_t i = 0; i < vals.size(); i++) {
         _values[i]->setVal(vals[i]);
@@ -961,7 +904,7 @@ void ArrayParamValue::setVal(string val, u_int32_t index)
     _values[index]->setVal(val);
 }
 
-void ArrayParamValue::pack(u_int8_t* buff, u_int32_t offset)
+void ArrayParamValue::pack(u_int8_t *buff, u_int32_t offset)
 {
     /* The FW expect a big endian buffer for both the order of the elements in the array
      * and the values of each element. So if elementsInDW = 2, _values[0] will point to buff[16..31]
@@ -969,7 +912,7 @@ void ArrayParamValue::pack(u_int8_t* buff, u_int32_t offset)
      * elementsInDW = 1, 2, 4
      */
     vector<ParamValue*>::iterator j;
-    for(vector<ParamValue*>::iterator it = _values.begin() ; it != _values.end();) {
+    for (vector<ParamValue*>::iterator it = _values.begin(); it != _values.end();) {
         int elementsInDW = 32 / _elementSizeInBits;
         j = it + elementsInDW - 1;
         unsigned int iterations = elementsInDW;
@@ -983,22 +926,10 @@ void ArrayParamValue::pack(u_int8_t* buff, u_int32_t offset)
     }
 }
 
-void ArrayParamValue::pack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-    pack(buff, bitOffset);
-
-}
-
-void ArrayParamValue::unpack(u_int8_t* buff, u_int32_t offset)
+void ArrayParamValue::unpack(u_int8_t *buff, u_int32_t offset)
 {
     vector<ParamValue*>::iterator j;
-    for(vector<ParamValue*>::iterator it = _values.begin(); it != _values.end();) {
+    for (vector<ParamValue*>::iterator it = _values.begin(); it != _values.end();) {
         int elementsInDW = 32 / _elementSizeInBits;
         j = it + elementsInDW - 1;
         unsigned int iterations = elementsInDW;
@@ -1012,17 +943,6 @@ void ArrayParamValue::unpack(u_int8_t* buff, u_int32_t offset)
     }
 }
 
-void ArrayParamValue::unpack(u_int8_t* buff, string offset)
-{
-    u_int32_t bitOffset;
-    u_int32_t pos;
-
-    pos = offset.find('.');
-    //only parse the number before the dot, because it is an array
-    bitOffset = 8 * strtol(offset.substr(0, pos).c_str(), NULL, 0);
-    unpack(buff, bitOffset);
-}
-
 void ArrayParamValue::parseValue(string strToParse, u_int32_t& val, string& strVal)
 {
     _values[0]->parseValue(strToParse, val, strVal);
@@ -1031,7 +951,7 @@ void ArrayParamValue::parseValue(string strToParse, u_int32_t& val, string& strV
 vector<u_int32_t> ArrayParamValue::getIntVals()
 {
     vector<u_int32_t> v;
-    for(vector<ParamValue*>::iterator it = _values.begin(); it != _values.end();it++) {
+    for (vector<ParamValue*>::iterator it = _values.begin(); it != _values.end(); it++) {
         v.push_back((*it)->getIntVal());
     }
     return v;
@@ -1040,16 +960,17 @@ vector<u_int32_t> ArrayParamValue::getIntVals()
 vector<string> ArrayParamValue::getStrVals()
 {
     vector<string> v;
-    for(vector<ParamValue*>::iterator it = _values.begin(); it != _values.end(); it++) {
+    for (vector<ParamValue*>::iterator it = _values.begin(); it != _values.end(); it++) {
         v.push_back((*it)->getVal());
     }
     return v;
 }
 
 EnumArrayParamValue::EnumArrayParamValue(string size, u_int32_t count, enum ParamType paramType,
-        map<string, u_int32_t> textualValues) : ArrayParamValue(size, count, paramType) {
-    for(u_int32_t i = 0; i < count; i++) {
-        ParamValue* t = new EnumParamValue(_elementSizeInBits, textualValues);
+                                         map<string, u_int32_t> textualValues) : ArrayParamValue(size, count, paramType)
+{
+    for (u_int32_t i = 0; i < count; i++) {
+        ParamValue *t = new EnumParamValue(_elementSizeInBits, textualValues);
         _values.push_back(t);
     }
 }
