@@ -47,16 +47,26 @@
 #include "mlxcfg_utils.h"
 #include "mlxcfg_generic_commander.h"
 
+#define DISABLE_SLOT_POWER_LIMITER  "DISABLE_SLOT_POWER_LIMITER"
+#define DISABLE_SLOT_POWER_LIMITER_WARN \
+    "\nWARNING: YOU ARE ABOUT TO RE-ENABLE ONE OR MORE NETWORK PORTS THAT HAVE BEEN SHUT DOWN DUE TO INSUFFICIENT POWER ON THE PCIE SLOT."\
+    "\nYOUR ACTION MAY RISK PROPER FUNCTIONAL OR PHYSICAL OPERATION OF THE CARD OR OF THE SERVER."\
+    "\nMELLANOX TECHNOLOGIES HEREBY DISCLAIMS ANY AND ALL LIABILITY RELATED TO ANY DAMAGE CAUSED AS A RESULT OF THIS RE-ENABLEMENT."\
+    "\nBY PROCEEDING WITH THIS RE-ENABLEMENT YOU WILL INVALIDATE ANY WARRANTY PROVIDED BY MELLANOX TECHNOLOGIES RELATED TO THIS CARD."\
+    "\nIT IS ADVISED TO CHECK THE CARD\'S USER MANUAL FOR POWER SPECIFICATIONS OR TO CONTACT MELLANOX SUPPORT BEFORE PERFORMING THIS ACTION."\
+    "\nARE YOU SURE YOU WANT TO CONTINUE? [Y/N] [N]: "
+
+
 #define NEXT_STR "Next Boot"
 #define DEFAULT_CURRENT_NOT_SUPPORTED_PREFIX "Device Firmware does not support reading "
 #define TLV_NAME_MAX_LENGTH 256
 #define EXIT_IF_RC_NOT_OK(rc) \
-    if (rc != MLX_CFG_OK) {\
-        return rc;\
+    if (rc != MLX_CFG_OK) { \
+        return rc; \
     }
 
 #define PRINT_SYNTAX_ERROR_AND_EXIT \
-    printf("-E- Syntax error in: %s\n", line->c_str());\
+    printf("-E- Syntax error in: %s\n", line->c_str()); \
     return MLX_CFG_ERROR;
 
 // Signal handler section
@@ -67,23 +77,23 @@ void TerminationHandler(int signum);
 
 static BOOL CtrlHandler( DWORD fdwCtrlType )
 {
-    switch( fdwCtrlType )
+    switch (fdwCtrlType)
     {
-      // Handle the CTRL-C signal.
-      case CTRL_C_EVENT:
-      // CTRL-CLOSE: confirm that the user wants to exit.
-      case CTRL_CLOSE_EVENT:
-      // Pass other signals to the next handler.
-      case CTRL_BREAK_EVENT:
-      case CTRL_LOGOFF_EVENT:
-      case CTRL_SHUTDOWN_EVENT:
-          TerminationHandler(SIGINT);
-          return TRUE;
+    // Handle the CTRL-C signal.
+    case CTRL_C_EVENT:
+    // CTRL-CLOSE: confirm that the user wants to exit.
+    case CTRL_CLOSE_EVENT:
+    // Pass other signals to the next handler.
+    case CTRL_BREAK_EVENT:
+    case CTRL_LOGOFF_EVENT:
+    case CTRL_SHUTDOWN_EVENT:
+        TerminationHandler(SIGINT);
+        return TRUE;
 
-      default:
+    default:
         return FALSE;
     }
- }
+}
 
 #endif
 
@@ -92,11 +102,11 @@ void TerminationHandler(int signum)
     static volatile sig_atomic_t fatal_error_in_progress = 0;
 
     if (fatal_error_in_progress) {
-        raise (signum);
+        raise(signum);
     }
     fatal_error_in_progress = 1;
 
-    signal (signum, SIG_DFL);
+    signal(signum, SIG_DFL);
     printf("\n Received signal %d.\n", signum);
     fflush(stdout);
     raise(signum);
@@ -115,9 +125,9 @@ void initHandler()
 #endif
 
     //set the signal handler
-    for (int i=0; i < SIGNAL_NUM ; i++) {
+    for (int i = 0; i < SIGNAL_NUM; i++) {
         void (*prevFunc)(int);
-        prevFunc = signal(signalList[i],TerminationHandler);
+        prevFunc = signal(signalList[i], TerminationHandler);
         if (prevFunc == SIG_ERR) {
             printf("-E- failed to set signal Handler.");
             exit(MLX_CFG_ERROR);
@@ -151,7 +161,7 @@ typedef struct QueryOutputItem {
     string strSetVal;
 } QueryOutputItem;
 
-inline void copyVal(QueryOutputItem&o , const ParamView& p, QueryType qT)
+inline void copyVal(QueryOutputItem&o, const ParamView& p, QueryType qT)
 {
     if (qT == QueryDefault) {
         o.defVal = p.val;
@@ -165,72 +175,80 @@ inline void copyVal(QueryOutputItem&o , const ParamView& p, QueryType qT)
     }
 }
 
-inline void copySet(QueryOutputItem&o , const ParamView& p)
+inline void copySet(QueryOutputItem&o, const ParamView& p)
 {
     o.setVal = p.val;
     o.strSetVal = p.strVal;
 }
 
-mlxCfgStatus MlxCfg::err(bool report, const char* fmt, ...)
+mlxCfgStatus MlxCfg::err(bool report, const char *fmt, ...)
 {
-     va_list  args;
-     va_start(args, fmt);
-     char errBuff[MAX_ERR_STR_LEN] = {0};
+    va_list args;
+    va_start(args, fmt);
+    char errBuff[MAX_ERR_STR_LEN] = {0};
 
-     if (vsnprintf(errBuff, MAX_ERR_STR_LEN, fmt, args) >= MAX_ERR_STR_LEN) {
-         strcpy(&errBuff[MAX_ERR_STR_LEN - 5], "...");
+    if (vsnprintf(errBuff, MAX_ERR_STR_LEN, fmt, args) >= MAX_ERR_STR_LEN) {
+        strcpy(&errBuff[MAX_ERR_STR_LEN - 5], "...");
     }
-     _errStr = errBuff;
-     if (report) {
-         fprintf(stdout,PRE_ERR_MSG" %s\n", _errStr.c_str());
-     }
-     va_end(args);
-     return MLX_CFG_ERROR;
+    _errStr = errBuff;
+    if (report) {
+        fprintf(stdout, PRE_ERR_MSG " %s\n", _errStr.c_str());
+    }
+    va_end(args);
+    return MLX_CFG_ERROR;
 }
 
 void MlxCfg::printErr()
 {
-    fprintf(stdout,PRE_ERR_MSG" %s\n", _errStr.c_str());
+    fprintf(stdout, PRE_ERR_MSG " %s\n", _errStr.c_str());
 }
 
 
-bool MlxCfg::askUser(const char* question)
+bool MlxCfg::askUser(const char *question, bool add_prefix, bool add_suffix)
 {
-if (question == NULL) {
-     printf("\n Do you want to continue ? (y/n) [n] : ");
- } else {
-     printf("\n %s ? (y/n) [n] : ", question);
- }
+    const char * prefix = "";
+    if(add_prefix){
+        prefix = "\n ";
+    }
 
- if (_mlxParams.yes)
-     printf("y\n");
- else {
-     fflush(stdout);
-     std::string answer;
-     std::getline(std::cin, answer);
+    const char * suffix = "";
+    if(add_suffix) {
+        suffix = " (y/n) [n] : ";
+    }
+    if (question == NULL) {
+        question = "Do you want to continue?";
+    }
+    printf("%s%s%s", prefix, question, suffix);
 
-     if (  strcasecmp(answer.c_str(), "y") &&
-           strcasecmp(answer.c_str(), "yes"))  {
+    if (_mlxParams.yes) {
+        printf("y\n");
+    } else {
+        fflush(stdout);
+        std::string answer;
+        std::getline(std::cin, answer);
 
-         err(false, "Aborted by user.");
-         return false;
-     }
- }
- return true;
+        if (strcasecmp(answer.c_str(), "y") &&
+            strcasecmp(answer.c_str(), "yes")) {
+
+            err(false, "Aborted by user.");
+            return false;
+        }
+    }
+    return true;
 }
 
 mlxCfgStatus MlxCfg::queryDevsCfg()
 {
-    bool shouldFail= false;
+    bool shouldFail = false;
     if (_mlxParams.device.length()) {
 
         if (queryDevCfg(_mlxParams.device.c_str())) {
             printErr();
-            shouldFail= true;
+            shouldFail = true;
         }
     } else {
-        int  numOfDev;
-        dev_info* dev = mdevices_info(MDEVS_TAVOR_CR, &numOfDev);
+        int numOfDev;
+        dev_info *dev = mdevices_info(MDEVS_TAVOR_CR, &numOfDev);
 
         if (dev == NULL) {
             return err(true, "Failed to get devices.");
@@ -241,17 +259,17 @@ mlxCfgStatus MlxCfg::queryDevsCfg()
         }
         //printf("-D- num of dev: %d , 1st dev : %s\n", numOfDev, buf);
         dev_info  *devPtr = dev;
-        char pcibuf[32]= {0};
+        char pcibuf[32] = {0};
 
-        for(int i=0 ; i < numOfDev ; i++) {
+        for (int i = 0; i < numOfDev; i++) {
 #ifdef __FREEBSD__
-            const char* device_name_ptrn = "pci%d:%d:%d:%d";
+            const char *device_name_ptrn = "pci%d:%d:%d:%d";
 #else
-            const char* device_name_ptrn = "%04x:%02x:%02x.%x";
+            const char *device_name_ptrn = "%04x:%02x:%02x.%x";
 #endif
-            snprintf(pcibuf,32, device_name_ptrn, devPtr->pci.domain, devPtr->pci.bus,\
-                    devPtr->pci.dev, devPtr->pci.func);
-            if (queryDevCfg(devPtr->pci.conf_dev, pcibuf, i+1)){
+            snprintf(pcibuf, 32, device_name_ptrn, devPtr->pci.domain, devPtr->pci.bus, \
+                     devPtr->pci.dev, devPtr->pci.func);
+            if (queryDevCfg(devPtr->pci.conf_dev, pcibuf, i + 1)) {
                 printErr();
                 shouldFail = true;
             }
@@ -259,17 +277,19 @@ mlxCfgStatus MlxCfg::queryDevsCfg()
         }
         mdevices_info_destroy(dev, numOfDev);
     }
-    return shouldFail? MLX_CFG_ERROR : MLX_CFG_OK;
+    return shouldFail ? MLX_CFG_ERROR : MLX_CFG_OK;
 }
 
 static void printValue(string strVal, u_int32_t val)
 {
     string s = numToStr(val);
     u_int32_t n;
-    if(strVal == "" || strVal == s || (strToNum(strVal, n) && n == val)) {
+    if (strVal == "" || strVal == s || (strToNum(strVal, n) && n == val)) {
         printf("%-16u", val);
     } else {
-        strVal += "(" + s + ")";
+        if (strVal.find("Array") == string::npos) {
+            strVal += "(" + s + ")";
+        }
         printf("%-16s", strVal.c_str());
     }
 }
@@ -277,21 +297,21 @@ static void printValue(string strVal, u_int32_t val)
 static void printParam(string param, u_int32_t val)
 {
     if (val == MLXCFG_UNKNOWN) {
-            printf("%-16s", "N/A");
-        } else {
-            printValue(param, val); 
-        }
+        printf("%-16s", "N/A");
+    } else {
+        printValue(param, val);
+    }
     return;
 }
 
-static void printOneParam(const char* name, QueryOutputItem o,
-        u_int8_t verbose, bool printNewCfg)
+static void printOneParam(const char *name, QueryOutputItem o,
+                          u_int8_t verbose, bool printNewCfg)
 {
     bool showDefault = QUERY_DEFAULT_MASK & verbose;
     bool showCurrent = QUERY_CURRENT_MASK & verbose;
 
     if ((showDefault && o.nextVal != o.defVal) ||
-            (showCurrent && o.nextVal != o.currVal)) {
+        (showCurrent && o.nextVal != o.currVal)) {
         printf("*        %-36s", name);
     } else {
         printf("         %-36s", name);
@@ -315,34 +335,47 @@ static void printOneParam(const char* name, QueryOutputItem o,
     return;
 
 }
-const char* MlxCfg::getDeviceName(const char* dev)
+
+const char* MlxCfg::getDeviceName(const char *dev)
 {
-    mfile* mf;
-    dm_dev_id_t dev_type;
-    u_int32_t dev_id;
+    mfile *mf;
     u_int32_t chip_rev;
+    u_int32_t _dev_id;
 
     mf = mopen(dev);
     if (!mf) {
         return "";
     }
-    if (dm_get_device_id(mf, &dev_type, &dev_id, &chip_rev)) {
+    if (dm_get_device_id(mf, &_devType, &_dev_id, &chip_rev)) {
         mclose(mf);
         return "";
     }
     mclose(mf);
-    return dm_dev_type2str(dev_type);
+    return dm_dev_type2str(_devType);
 }
 
-void MlxCfg::printOpening(const char* dev, int devIndex)
+void MlxCfg::printOpening(const char *dev, int devIndex)
 {
     printf("\nDevice #%d:\n", devIndex);
     printf("----------\n\n");
-    printf("%-16s%-16s\n", "Device type:", getDeviceName(dev));
-    printf("%-16s%-16s\n", "PCI device:", dev);
-    // TODO : get this info
-    //printf("%-16s%-16s\n", "Part Number:", "123456");
-    //printf("%-16s%-16s\n", "Psid:", "78901111");
+    const char *devType = getDeviceName(dev);
+    printf("%-16s%-16s\n", "Device type:", devType);
+    if ((_devType != DeviceConnectX3 && _devType != DeviceConnectX3Pro && _devType != DeviceConnectX2)) {
+        std::vector<char> info;
+        info.reserve(16);
+        if (!getDeviceInformationString(dev, Device_Name, info)) {
+            printf("%-16s%-16s\n", "Name:", "N/A");
+        } else {
+            printf("%-16s%-16s\n", "Name:", info.data());
+        }
+        info.clear();
+        if (!getDeviceInformationString(dev, Device_Description, info)) {
+            printf("%-16s%-16s\n", "Description:", "N/A");
+        } else {
+            printf("%-16s%-16s\n", "Description:", info.data());
+        }
+    }
+    printf("%-16s%-16s\n", "Device:", dev);
 }
 
 void MlxCfg::printConfHeader(bool showDefualt, bool showNew, bool showCurrent)
@@ -350,16 +383,16 @@ void MlxCfg::printConfHeader(bool showDefualt, bool showNew, bool showCurrent)
     //print configuration Header
     if (showDefualt) {
         if (showCurrent) {
-            printf("%-16s%36s%16s%18s","Configurations:","Default","Current",
-                    NEXT_STR);
+            printf("%-16s%36s%16s%18s", "Configurations:", "Default", "Current",
+                   NEXT_STR);
         } else {
-            printf("%-16s%36s%18s","Configurations:","Default", NEXT_STR);
+            printf("%-16s%36s%18s", "Configurations:", "Default", NEXT_STR);
         }
     } else {
         if (showCurrent) {
-            printf("%-16s%36s%18s","Configurations:","Current", NEXT_STR);
+            printf("%-16s%36s%18s", "Configurations:", "Current", NEXT_STR);
         } else {
-            printf("%-16s%38s","Configurations:", NEXT_STR);
+            printf("%-16s%38s", "Configurations:", NEXT_STR);
         }
     }
     if (showNew) {
@@ -368,7 +401,7 @@ void MlxCfg::printConfHeader(bool showDefualt, bool showNew, bool showCurrent)
 }
 
 void prepareSetInput(vector<QueryOutputItem>& output,
-        vector<ParamView>& params)
+                     vector<ParamView>& params)
 {
     VECTOR_ITERATOR(ParamView, params, p) {
         QueryOutputItem o;
@@ -382,7 +415,7 @@ void prepareSetInput(vector<QueryOutputItem>& output,
 }
 
 void prepareQueryOutput(vector<QueryOutputItem>& output,
-        vector<ParamView>& params, QueryType qT)
+                        vector<ParamView>& params, QueryType qT)
 {
     VECTOR_ITERATOR(ParamView, params, p) {
         bool found = false;
@@ -393,31 +426,45 @@ void prepareQueryOutput(vector<QueryOutputItem>& output,
                 break;
             }
         }
-        if(!found) {
+        if (!found) {
             QueryOutputItem o;
             o.mlxconfigName = p->mlxconfigName;
             o.defVal = MLXCFG_UNKNOWN;
             o.currVal = MLXCFG_UNKNOWN;
             o.nextVal = MLXCFG_UNKNOWN;
+            o.setVal = MLXCFG_UNKNOWN;
             copyVal(o, *p, qT);
             output.push_back(o);
         }
     }
 }
 
-void queryAux(Commander* commander, vector<ParamView>& params,
-        vector<ParamView>& setParams, bool isSetCmd, QueryType qT)
+void queryAux(Commander *commander, vector<ParamView>& params,
+              vector<ParamView>& paramsToQuery, vector<string>& failedTLVs, QueryType qT)
 {
-    if (isSetCmd) {
-        params = setParams;
+    if (paramsToQuery.size() != 0) {
+        params = paramsToQuery;
         commander->queryParamViews(params, qT);
     } else {
-        commander->queryAll(params, qT);
+        commander->queryAll(params, failedTLVs, qT);
     }
 }
 
-mlxCfgStatus MlxCfg::queryDevCfg(Commander* commander, const char* dev,const char* pci,
-        int devIndex, bool printNewCfg)
+string checkFailedTLVsVector(const vector<string>& failedTLVs, string queryType)
+{
+    string failedTLVsErrorMessage = "";
+    if (failedTLVs.size() != 0) {
+        failedTLVsErrorMessage += "    Failed to query the " + queryType + " values of the following TLVs:\n    ";
+        CONST_VECTOR_ITERATOR(string, failedTLVs, it) {
+            failedTLVsErrorMessage += (*it) + " ";
+        }
+        failedTLVsErrorMessage += "\n";
+    }
+    return failedTLVsErrorMessage;
+}
+
+mlxCfgStatus MlxCfg::queryDevCfg(Commander *commander, const char *dev, const char *pci,
+                                 int devIndex, bool printNewCfg)
 {
     (void) pci;
     std::vector<QueryOutputItem> output;
@@ -429,6 +476,8 @@ mlxCfgStatus MlxCfg::queryDevCfg(Commander* commander, const char* dev,const cha
     printOpening(dev, devIndex);
     printf("\n");
 
+    vector<string> defaultFailedTLVs, currentFailedTLVs, nextFailedTLVs;
+
     try {
 
         if (_mlxParams.enableVerbosity) {
@@ -454,14 +503,12 @@ mlxCfgStatus MlxCfg::queryDevCfg(Commander* commander, const char* dev,const cha
         }
 
         if (showDefault) {
-            queryAux(commander, defaultParams, _mlxParams.setParams,
-                    printNewCfg, QueryDefault);
+            queryAux(commander, defaultParams, _mlxParams.setParams, defaultFailedTLVs, QueryDefault);
         }
         if (showCurrent) {
-            queryAux(commander, currentParams, _mlxParams.setParams,
-                    printNewCfg, QueryCurrent);
+            queryAux(commander, currentParams, _mlxParams.setParams, currentFailedTLVs, QueryCurrent);
         }
-        queryAux(commander, params, _mlxParams.setParams, printNewCfg, QueryNext);
+        queryAux(commander, params, _mlxParams.setParams, nextFailedTLVs, QueryNext);
     } catch (MlxcfgException& e) {
         err(false, "%s", e._err.c_str());
         return MLX_CFG_ERROR_EXIT;
@@ -474,34 +521,39 @@ mlxCfgStatus MlxCfg::queryDevCfg(Commander* commander, const char* dev,const cha
         prepareQueryOutput(output, currentParams, QueryCurrent);
     }
     prepareQueryOutput(output, params, QueryNext);
-    
+
     //print output table:
     VECTOR_ITERATOR(QueryOutputItem, output, o) {
         printOneParam(o->mlxconfigName.c_str(),
-                *o,
-                QUERY_NEXT_MASK |
-                    (showDefault ? QUERY_DEFAULT_MASK :0) |
-                    (showCurrent ? QUERY_CURRENT_MASK : 0),
-                printNewCfg);
+                      *o,
+                      QUERY_NEXT_MASK |
+                      (showDefault ? QUERY_DEFAULT_MASK : 0) |
+                      (showCurrent ? QUERY_CURRENT_MASK : 0),
+                      printNewCfg);
         isParamsDiffer |= (showDefault && (o->nextVal != o->defVal))
-                || (showCurrent && (o->nextVal != o->currVal));
+                          || (showCurrent && (o->nextVal != o->currVal));
     }
 
+    string failedTLVsErrorMessage = "Failed to query some of the TLVs:\n";
+    failedTLVsErrorMessage += checkFailedTLVsVector(defaultFailedTLVs, "default");
+    failedTLVsErrorMessage += checkFailedTLVsVector(currentFailedTLVs, "current");
+    failedTLVsErrorMessage += checkFailedTLVsVector(nextFailedTLVs, "next");
+
     if (isParamsDiffer) {
         printf("The '*' shows parameters with next value different "
-                "from default/current value.\n");
+               "from default/current value.\n");
     }
 
     if (_mlxParams.enableVerbosity) {
         if (!defaultSupported && !currentSupported) {
             printf(DEFAULT_CURRENT_NOT_SUPPORTED_PREFIX
-                    "default and current configurations\n");
+                   "default and current configurations\n");
         } else if (!defaultSupported) {
             printf(DEFAULT_CURRENT_NOT_SUPPORTED_PREFIX
-                    "default configurations\n");
+                   "default configurations\n");
         } else if (!currentSupported) {
             printf(DEFAULT_CURRENT_NOT_SUPPORTED_PREFIX
-                    "current configurations\n");
+                   "current configurations\n");
         }
     }
 
@@ -510,14 +562,20 @@ mlxCfgStatus MlxCfg::queryDevCfg(Commander* commander, const char* dev,const cha
         return MLX_CFG_ERROR_EXIT;
     }
 
+    if (defaultFailedTLVs.size() != 0 ||
+        currentFailedTLVs.size() != 0 || nextFailedTLVs.size() != 0) {
+        err(false, "%s", failedTLVsErrorMessage.c_str());
+        return MLX_CFG_ERROR;
+    }
+
     return failedToGetCfg ? MLX_CFG_ERROR : MLX_CFG_OK;
 }
 
-mlxCfgStatus MlxCfg::queryDevCfg(const char* dev,const char* pci,
-        int devIndex, bool printNewCfg)
+mlxCfgStatus MlxCfg::queryDevCfg(const char *dev, const char *pci,
+                                 int devIndex, bool printNewCfg)
 {
     mlxCfgStatus rc;
-    Commander* commander = NULL;
+    Commander *commander = NULL;
     try {
         commander = Commander::create(string(dev), _mlxParams.dbName);
     } catch (MlxcfgException& e) {
@@ -530,22 +588,45 @@ mlxCfgStatus MlxCfg::queryDevCfg(const char* dev,const char* pci,
     return rc;
 }
 
+const char * MlxCfg::getConfigWarning(const string & mlx_config_name,
+        const string & set_val)
+{
+    if(mlx_config_name == DISABLE_SLOT_POWER_LIMITER) {
+        if(set_val == "1" || strcasecmp(set_val.c_str(), "true") == 0) {
+            return DISABLE_SLOT_POWER_LIMITER_WARN;
+        }
+    }
+    return NULL;
+}
+
 mlxCfgStatus MlxCfg::setDevCfg()
 {
-    Commander* commander = NULL;
+    Commander *commander = NULL;
 
     try {
         commander = Commander::create(string(_mlxParams.device),
-                _mlxParams.dbName);
+                                      _mlxParams.dbName);
     } catch (MlxcfgException& e) {
         delete commander;
         err(false, "%s", e._err.c_str());
         printErr();
         return MLX_CFG_ERROR;
     }
+    // check if there is a set of DISABLE_SLOT_POWER
+    //for mlx_config_name
+    VECTOR_ITERATOR(ParamView, _mlxParams.setParams, p) {
+        const char * warning_msg = getConfigWarning(p->mlxconfigName, p->setVal);
+        if(warning_msg){
+            if(!askUser(warning_msg, false, false)) {
+                delete commander;
+                printErr();
+                return MLX_CFG_ABORTED;
+            }
+        }
+    }
 
     if (queryDevCfg(commander, _mlxParams.device.c_str(), NULL, 1, true)
-            == MLX_CFG_ERROR_EXIT){
+        == MLX_CFG_ERROR_EXIT) {
         delete commander;
         printErr();
         return MLX_CFG_ERROR;
@@ -559,7 +640,7 @@ mlxCfgStatus MlxCfg::setDevCfg()
         printf("-W- Incorrect configuration might yield unexpected results. running in this mode is not recommended.");
     }
     // ask user
-    if(!askUser("Apply new Configuration?")) {
+    if (!askUser("Apply new Configuration?")) {
         delete commander;
         printErr();
         return MLX_CFG_ABORTED;
@@ -569,7 +650,7 @@ mlxCfgStatus MlxCfg::setDevCfg()
         printf("Applying... ");
         commander->setCfg(_mlxParams.setParams, _mlxParams.force);
         printf("Done!\n");
-        const char* resetStr = commander->loadConfigurationGetStr();
+        const char *resetStr = commander->loadConfigurationGetStr();
         printf("-I- %s\n", resetStr);
         delete commander;
         return MLX_CFG_OK;
@@ -586,8 +667,8 @@ mlxCfgStatus MlxCfg::resetDevsCfg()
     // check if a single device was specified and apply reset for this device only
     if (_mlxParams.device.length()) {
         char buff[256] = {0};
-        snprintf(buff,256,"Reset configuration for device %s? ", _mlxParams.device.c_str());
-        if (!askUser(buff)){
+        snprintf(buff, 256, "Reset configuration for device %s?", _mlxParams.device.c_str());
+        if (!askUser(buff)) {
             printErr();
             return MLX_CFG_ABORTED;
         }
@@ -600,8 +681,8 @@ mlxCfgStatus MlxCfg::resetDevsCfg()
         }
     } else {
         // reset all devices.
-        int  numOfDev;
-        dev_info* dev = mdevices_info(MDEVS_TAVOR_CR, &numOfDev);
+        int numOfDev;
+        dev_info *dev = mdevices_info(MDEVS_TAVOR_CR, &numOfDev);
 
         if (dev == NULL) {
             return err(true, "Failed to get devices.");
@@ -610,7 +691,7 @@ mlxCfgStatus MlxCfg::resetDevsCfg()
             mdevices_info_destroy(dev, numOfDev);
             return err(true, NO_DEV_ERR);
         }
-        if (!askUser("Reset configuration for all devices? ")){
+        if (!askUser("Reset configuration for all devices?")) {
             printErr();
             mdevices_info_destroy(dev, numOfDev);
             return MLX_CFG_ABORTED;
@@ -618,10 +699,10 @@ mlxCfgStatus MlxCfg::resetDevsCfg()
         printf("Applying... ");
 
         dev_info *devPtr = dev;
-        bool shouldFail= false;
+        bool shouldFail = false;
 
-        for(int i=0 ; i < numOfDev ; i++) {
-            if (resetDevCfg(devPtr->pci.conf_dev)){
+        for (int i = 0; i < numOfDev; i++) {
+            if (resetDevCfg(devPtr->pci.conf_dev)) {
                 shouldFail = true;
             }
             devPtr++;
@@ -642,23 +723,23 @@ mlxCfgStatus MlxCfg::resetDevsCfg()
 
 mlxCfgStatus MlxCfg::clrDevSem()
 {
-    Commander* commander = NULL;
+    Commander *commander = NULL;
     mlxCfgStatus rc = MLX_CFG_OK;
 
     printf("-W- Forcefully clearing device Semaphore...\n");
     try {
         commander = Commander::create(_mlxParams.device, _mlxParams.dbName);
         commander->clearSemaphore();
-    } catch(MlxcfgException& e){
+    } catch (MlxcfgException& e) {
         printf("-E- %s\n", e._err.c_str());
         rc = MLX_CFG_ERROR;
     }
 
-    if(!commander) {
+    if (commander != NULL) {
         delete commander;
     }
 
-    if(rc == MLX_CFG_OK) {
+    if (rc == MLX_CFG_OK) {
         printf(" Done!\n");
     } else {
         printf(" Failed!\n");
@@ -669,7 +750,7 @@ mlxCfgStatus MlxCfg::clrDevSem()
 
 mlxCfgStatus MlxCfg::setDevRawCfg()
 {
-    Commander* commander = NULL;
+    Commander *commander = NULL;
     try {
         commander = Commander::create(_mlxParams.device, _mlxParams.dbName);
         // open file
@@ -709,8 +790,9 @@ mlxCfgStatus MlxCfg::setDevRawCfg()
             printf("Raw TLV #%d Info:\n%s\n", tlvIdx, dumpStr.c_str());
         }
         // ask user
-        if(!askUser("Operation intended for advanced users.\n Are you sure you want to apply raw TLV file?")) {
+        if (!askUser("Operation intended for advanced users.\n Are you sure you want to apply raw TLV file?")) {
             printErr();
+            delete commander;
             return MLX_CFG_ABORTED;
         }
         printf("Applying... ");
@@ -719,11 +801,17 @@ mlxCfgStatus MlxCfg::setDevRawCfg()
         for (std::vector<std::vector<u_int32_t> >::iterator it = rawTlvsAsDw.begin(); it != rawTlvsAsDw.end(); it++, tlvIdx++) {
             commander->setRawCfg(*it);
         }
-    } catch(MlxcfgException& e) {
+        //send mfrl command to fw
+        //this command indicate to the fw that next time perst signal go down
+        //[reboot] fw need to perform reset )
+        commander->loadConfigurationGetStr();
+    } catch (MlxcfgException& e) {
         delete commander;
         return err(true, "Failed to run set_raw command: %s", e._err.c_str());
     }
-    delete commander;
+    if (commander != NULL) {
+        delete commander;
+    }
     // done successfully
     printf("Done!\n");
     printf("-I- Please reboot machine to load new configurations.\n");
@@ -732,8 +820,8 @@ mlxCfgStatus MlxCfg::setDevRawCfg()
 
 mlxCfgStatus MlxCfg::backupCfg()
 {
-    FILE * file;
-    Commander* commander = NULL;
+    FILE *file;
+    Commander *commander = NULL;
     vector<BackupView> views;
 
     try {
@@ -741,9 +829,9 @@ mlxCfgStatus MlxCfg::backupCfg()
         printf("Collecting...\n");
         commander->backupCfgs(views);
         delete commander;
-    } catch(MlxcfgException& e) {
+    } catch (MlxcfgException& e) {
         printf(" Failed!\n");
-        if(commander) {
+        if (commander) {
             delete commander;
         }
         return err(true, "Failed to backup the configurations: %s", e._err.c_str());
@@ -762,8 +850,8 @@ mlxCfgStatus MlxCfg::backupCfg()
 
         fprintf(file, "%s\n", MLNX_RAW_TLV_FILE_SIG);
 
-        for(std::vector<BackupView>::iterator it = views.begin();
-                it != views.end(); it++) {
+        for (std::vector<BackupView>::iterator it = views.begin();
+             it != views.end(); it++) {
             fprintf(file, "%% TLV Type: 0x%08x, Writer ID: %s(0x%02x)"
                     ", Writer Host ID: 0x%02x\n",
                     it->type,
@@ -771,7 +859,7 @@ mlxCfgStatus MlxCfg::backupCfg()
                     it->writerId,
                     it->writerHostId);
             vector<u_int8_t> v = it->tlvBin;
-            for(size_t i = 0; i < v.size() / 4; i++) {
+            for (size_t i = 0; i < v.size() / 4; i++) {
                 fprintf(file, "0x%08x ", __cpu_to_be32(((u_int32_t*)v.data())[i]));
             }
             fprintf(file, "\n");
@@ -784,11 +872,12 @@ mlxCfgStatus MlxCfg::backupCfg()
     return MLX_CFG_OK;
 }
 
-mlxCfgStatus MlxCfg::tlvLine2DwVec(const std::string& tlvStringLine, std::vector<u_int32_t>& tlvVec) {
+mlxCfgStatus MlxCfg::tlvLine2DwVec(const std::string& tlvStringLine, std::vector<u_int32_t>& tlvVec)
+{
     tlvVec.resize(0);
     std::string dwStr;
     u_int32_t dw;
-    char* p = NULL;
+    char *p = NULL;
     std::istringstream isstm(tlvStringLine);
     while (isstm >> dwStr) {
         dw = strtoul(dwStr.c_str(), &p, 0);
@@ -800,9 +889,9 @@ mlxCfgStatus MlxCfg::tlvLine2DwVec(const std::string& tlvStringLine, std::vector
     return MLX_CFG_OK;
 }
 
-mlxCfgStatus MlxCfg::resetDevCfg(const char* dev)
+mlxCfgStatus MlxCfg::resetDevCfg(const char *dev)
 {
-    Commander* commander = NULL;
+    Commander *commander = NULL;
     mlxCfgStatus rc = MLX_CFG_OK;
 
     try {
@@ -814,7 +903,7 @@ mlxCfgStatus MlxCfg::resetDevCfg(const char* dev)
         rc = MLX_CFG_ERROR;
     }
 
-    if(commander) {
+    if (commander) {
         delete commander;
     }
 
@@ -823,7 +912,7 @@ mlxCfgStatus MlxCfg::resetDevCfg(const char* dev)
 
 mlxCfgStatus MlxCfg::showDevConfs()
 {
-    Commander* commander = NULL;
+    Commander *commander = NULL;
     mlxCfgStatus rc = MLX_CFG_OK;
 
     try {
@@ -835,27 +924,25 @@ mlxCfgStatus MlxCfg::showDevConfs()
         rc = MLX_CFG_ERROR;
     }
 
-    if(commander) {
+    if (commander) {
         delete commander;
     }
 
     return rc;
 }
 
-mlxCfgStatus MlxCfg::readNVInputFile(vector<u_int32_t>& buff)
+mlxCfgStatus MlxCfg::readBinFile(string fileName, vector<u_int32_t>& buff)
 {
     streampos size;
 
-    std::ifstream ifs(_mlxParams.NVInputFile.c_str(),
-            ios::in|ios::binary|ios::ate);
+    std::ifstream ifs(fileName.c_str(),
+                      ios::in | ios::binary | ios::ate);
     if (ifs.fail()) {
-        return err(true, "Failed to open file: %s",
-                _mlxParams.NVInputFile.c_str());
+        return err(true, "Failed to open file: %s", fileName.c_str());
     }
     size = ifs.tellg();
-    if (size%4 != 0) {
-        return err(true, "File %s is not DW aligned",
-                _mlxParams.NVInputFile.c_str());
+    if (size % 4 != 0) {
+        return err(true, "File %s is not DW aligned", fileName.c_str());
     }
     buff.resize((size_t)size / 4);
     ifs.seekg(0, ios::beg);
@@ -864,12 +951,17 @@ mlxCfgStatus MlxCfg::readNVInputFile(vector<u_int32_t>& buff)
     return MLX_CFG_OK;
 }
 
+mlxCfgStatus MlxCfg::readNVInputFile(vector<u_int32_t>& buff)
+{
+    return readBinFile(_mlxParams.NVInputFile, buff);
+}
+
 mlxCfgStatus MlxCfg::readNVInputFile(vector<string>& lines)
 {
     std::ifstream ifs(_mlxParams.NVInputFile.c_str());
     if (ifs.fail()) {
         return err(true, "Failed to open file: %s",
-                _mlxParams.NVInputFile.c_str());
+                   _mlxParams.NVInputFile.c_str());
     }
     for (std::string line; std::getline(ifs, line);) {
         lines.push_back(line);
@@ -894,15 +986,15 @@ mlxCfgStatus MlxCfg::readNVInputFile(string& content)
 
 mlxCfgStatus MlxCfg::writeNVOutputFile(vector<u_int32_t> content)
 {
-    FILE* file = fopen(_mlxParams.NVOutputFile.c_str(), "wb");
+    FILE *file = fopen(_mlxParams.NVOutputFile.c_str(), "wb");
     if (!file) {
         return err(true, "Failed to open file: %s",
-                _mlxParams.NVOutputFile.c_str());
+                   _mlxParams.NVOutputFile.c_str());
     }
-    for (u_int32_t i = 0 ; i < content.size() * 4; i += 4) {
-        size_t cnt = fwrite(((u_int8_t*)content.data()) + i ,1 , 4 , file);
-        (void)cnt; // avoid annoying unused return code/ unused variable in certain compilers
-        if (ferror(file)) {
+    for (u_int32_t i = 0; i < content.size() * 4; i += 4) {
+        const size_t count = 4;
+        size_t writtenCount = fwrite(((u_int8_t*)content.data()) + i, 1, count, file);
+        if (writtenCount != count || ferror(file)) {
             fclose(file);
             return err(true, "Write failed: %s.", strerror(errno));
         }
@@ -913,10 +1005,10 @@ mlxCfgStatus MlxCfg::writeNVOutputFile(vector<u_int32_t> content)
 
 mlxCfgStatus MlxCfg::writeNVOutputFile(string content)
 {
-    FILE* file = fopen(_mlxParams.NVOutputFile.c_str(), "w");
+    FILE *file = fopen(_mlxParams.NVOutputFile.c_str(), "w");
     if (!file) {
         return err(true, "Failed to open file: %s",
-                _mlxParams.NVOutputFile.c_str());
+                   _mlxParams.NVOutputFile.c_str());
     }
     fprintf(file, "%s", content.c_str());
     fclose(file);
@@ -942,10 +1034,10 @@ mlxCfgStatus MlxCfg::genTLVsFile()
         GenericCommander commander(NULL, _mlxParams.dbName);
         commander.genTLVsList(tlvs);
         VECTOR_ITERATOR(string, tlvs, it) {
-            sprintf(buff, "%-40s0\n", it->c_str());
+            sprintf(buff, "%-50s0\n", it->c_str());
             (*it) = buff;
         }
-    } catch(MlxcfgException& e) {
+    } catch (MlxcfgException& e) {
         printf("-E- %s\n", e._err.c_str());
         rc = MLX_CFG_ERROR;
     }
@@ -973,16 +1065,18 @@ mlxCfgStatus MlxCfg::genXMLTemplate()
     VECTOR_ITERATOR(string, lines, line) {
         string tlvName;
         string tLine = mlxcfg_trim(*line);
-        for(unsigned i = 0; i < tLine.size(); i++) {
+        for (unsigned i = 0; i < tLine.size(); i++) {
             if (tLine[i] != ' ' && tLine[i] != '\t') {
                 tlvName += tLine[i];
             } else {
                 while ((tLine[i] == ' ' || tLine[i] == '\t')
-                        && (++i < tLine.size())) { };
+                       && (++i < tLine.size())) {
+                }
+                ;
                 if ((i == tLine.size()) ||
-                        (tLine[i] != '0' && tLine[i] != '1') ||
-                        (i != tLine.size() - 1) ||
-                        tlvName.empty()) {
+                    (tLine[i] != '0' && tLine[i] != '1') ||
+                    (i != tLine.size() - 1) ||
+                    tlvName.empty()) {
                     PRINT_SYNTAX_ERROR_AND_EXIT
                 }
                 if (tLine[i] == '1') {
@@ -1004,7 +1098,7 @@ mlxCfgStatus MlxCfg::genXMLTemplate()
     try {
         GenericCommander commander(NULL, _mlxParams.dbName);
         commander.genXMLTemplate(tlvs, xmlTemplate, _mlxParams.allAttrs);
-    } catch(MlxcfgException& e) {
+    } catch (MlxcfgException& e) {
         printf("-E- %s\n", e._err.c_str());
         rc = MLX_CFG_ERROR;
     }
@@ -1047,7 +1141,7 @@ mlxCfgStatus MlxCfg::raw2XMLAux(bool isBin)
                 commander.raw2XML(lines, xmlTemplate);
             }
         }
-    } catch(MlxcfgException& e) {
+    } catch (MlxcfgException& e) {
         printf("-E- %s\n", e._err.c_str());
         rc = MLX_CFG_ERROR;
     }
@@ -1080,11 +1174,11 @@ mlxCfgStatus MlxCfg::XML2RawAux(bool isBin)
     try {
         GenericCommander commander(NULL, _mlxParams.dbName);
         if (isBin) {
-            //commander.XML2Bin(xml, binBuff);
+            commander.XML2Bin(xml, binBuff, false);
         } else {
             commander.XML2Raw(xml, raw);
         }
-    } catch(MlxcfgException& e) {
+    } catch (MlxcfgException& e) {
         printf("-E- %s\n", e._err.c_str());
         rc = MLX_CFG_ERROR;
     }
@@ -1109,8 +1203,82 @@ mlxCfgStatus MlxCfg::XML2Raw()
     return XML2RawAux(false);
 }
 
+mlxCfgStatus MlxCfg::XML2Bin()
+{
+    return XML2RawAux(true);
+}
 
-mlxCfgStatus MlxCfg::execute(int argc, char* argv[])
+mlxCfgStatus MlxCfg::createConf()
+{
+#if defined(NO_OPEN_SSL)
+    printf("-E- Not Implemented");
+    return MLX_CFG_ERROR_EXIT;
+#else
+    string xml;
+    vector<u_int32_t> buff;
+    mlxCfgStatus rc = MLX_CFG_OK;
+
+    rc = readNVInputFile(xml);
+    EXIT_IF_RC_NOT_OK(rc)
+
+    try {
+        GenericCommander commander(NULL, _mlxParams.dbName);
+        commander.createConf(xml, buff);
+        if (!_mlxParams.privPemFile.empty() && !_mlxParams.keyPairUUID.empty()) {
+            commander.sign(buff, _mlxParams.privPemFile, _mlxParams.keyPairUUID);
+        } else {
+            commander.sign(buff);
+        }
+    } catch (MlxcfgException& e) {
+        printf("-E- %s\n", e._err.c_str());
+        rc = MLX_CFG_ERROR;
+    }
+
+    if (rc == MLX_CFG_OK) {
+        printf("Saving output...\n");
+        rc = writeNVOutputFile(buff);
+        if (rc == MLX_CFG_OK) {
+            printf("Done!\n");
+        }
+    }
+
+    return rc;
+#endif
+}
+
+mlxCfgStatus MlxCfg::apply()
+{
+    vector<u_int32_t> buff;
+    vector<u_int8_t> bytesBuff;
+    Commander *commander = NULL;
+    mlxCfgStatus rc = MLX_CFG_OK;
+
+    rc = readNVInputFile(buff);
+    EXIT_IF_RC_NOT_OK(rc)
+
+    copyDwVectorToBytesVector(buff, bytesBuff);
+
+    try {
+        commander = Commander::create(_mlxParams.device, _mlxParams.dbName);
+        printf("Applying...\n");
+        ((GenericCommander*)commander)->apply(bytesBuff);
+    } catch (MlxcfgException& e) {
+        printf("-E- %s\n", e._err.c_str());
+        rc = MLX_CFG_ERROR;
+    }
+
+    if (commander != NULL) {
+        delete commander;
+    }
+
+    if (rc == MLX_CFG_OK) {
+        printf("Done!\n");
+    }
+
+    return rc;
+}
+
+mlxCfgStatus MlxCfg::execute(int argc, char *argv[])
 {
     mlxCfgStatus rc = parseArgs(argc, argv);
     if (rc) {
@@ -1127,36 +1295,57 @@ mlxCfgStatus MlxCfg::execute(int argc, char* argv[])
     case Mc_ShowConfs:
         ret = showDevConfs();
         break;
+
     case Mc_Query:
         ret = queryDevsCfg();
         break;
+
     case Mc_Set:
         ret = setDevCfg();
         break;
+
     case Mc_Reset:
         ret = resetDevsCfg();
         break;
+
     case Mc_Clr_Sem:
         ret = clrDevSem();
         break;
+
     case Mc_Set_Raw:
         ret = setDevRawCfg();
         break;
+
     case Mc_Backup:
         ret = backupCfg();
         break;
+
     case Mc_GenTLVsFile:
         ret = genTLVsFile();
         break;
+
     case Mc_GenXMLTemplate:
         ret = genXMLTemplate();
         break;
+
     case Mc_Raw2XML:
         ret = raw2XML();
         break;
+
     case Mc_XML2Raw:
         ret = XML2Raw();
         break;
+
+    case Mc_XML2Bin:
+        ret = XML2Bin();
+        break;
+    case Mc_CreateConf:
+        ret = createConf();
+        break;
+    case Mc_Apply:
+        ret = apply();
+        break;
+
     default:
         // should not reach here.
         return err(true, "invalid command.");
@@ -1164,13 +1353,13 @@ mlxCfgStatus MlxCfg::execute(int argc, char* argv[])
     return ret;
 }
 
-int main(int argc, char* argv[])
+int main(int argc, char *argv[])
 {
     try
     {
-    initHandler();
-    MlxCfg mc;
-    return mc.execute(argc, argv);
+        initHandler();
+        MlxCfg mc;
+        return mc.execute(argc, argv);
     }
     catch (std::exception& e)
     {
