@@ -70,9 +70,9 @@ class MlxCfgParams
 {
 public:
     MlxCfgParams() : device(), rawTlvFile(), NVInputFile(), NVOutputFile(),
-                     dbName(DB_NAME), privPemFile(), keyPairUUID(),
-                     allAttrs(false), cmd(Mc_UnknownCmd), yes(false),
-                     force(false), enableVerbosity(false) {}
+        dbName(DB_NAME), privPemFile(), keyPairUUID(),
+        allAttrs(false), cmd(Mc_UnknownCmd), yes(false),
+        force(false), enableVerbosity(false) {}
     ~MlxCfgParams() {}
 
     std::string device;
@@ -94,9 +94,9 @@ public:
 class MlxCfg
 {
 public:
-    MlxCfg() : _mlxParams(), _errStr(), _allInfo(){}
+    MlxCfg() : _mlxParams(), _errStr(), _allInfo(), _devType(DeviceUnknown) {}
     ~MlxCfg() {};
-    mlxCfgStatus execute(int argc, char* argv[]);
+    mlxCfgStatus execute(int argc, char *argv[]);
 private:
 
     // User interface and parsing methods
@@ -104,28 +104,31 @@ private:
     mlxCfgStatus showDevConfs();
     void printVersion();
     void printUsage();
-    void printOpening(const char* dev, int devIndex);
+    void printOpening(const char *dev, int devIndex);
     void printConfHeader(bool showDefualt, bool showNew, bool showCurrent);
-    mlxCfgStatus parseArgs(int argc, char* argv[]);
+    mlxCfgStatus parseArgs(int argc, char *argv[]);
     //Helper functions for parse args
-    mlxCfgStatus extractNVInputFile(int argc, char* argv[]);
-    mlxCfgStatus extractNVOutputFile(int argc, char* argv[]);
-    mlxCfgStatus extractSetCfgArgs(int argc, char* argv[]);
-    mlxCfgStatus extractQueryCfgArgs(int argc, char* argv[]);
+    mlxCfgStatus extractNVInputFile(int argc, char *argv[]);
+    mlxCfgStatus extractNVOutputFile(int argc, char *argv[]);
+    mlxCfgStatus extractSetCfgArgs(int argc, char *argv[]);
+    mlxCfgStatus extractQueryCfgArgs(int argc, char *argv[]);
+
+    const char * getConfigWarning(const string & mlx_config_name,
+            const string & set_val);
 
     bool tagExsists(string tag);
-    const char* getDeviceName(const char* dev);
+    const char* getDeviceName(const char *dev);
 
     // Query cmd
     mlxCfgStatus queryDevsCfg();
-    mlxCfgStatus queryDevCfg(const char* dev, const char* pci=(const char*)NULL, int devIndex=1, bool printNewCfg=false);
-    mlxCfgStatus queryDevCfg(Commander* commander, const char* dev, const char* pci=(const char*)NULL, int devIndex=1, bool printNewCfg=false);
+    mlxCfgStatus queryDevCfg(const char *dev, const char *pci = (const char*)NULL, int devIndex = 1, bool printNewCfg = false);
+    mlxCfgStatus queryDevCfg(Commander *commander, const char *dev, const char *pci = (const char*)NULL, int devIndex = 1, bool printNewCfg = false);
 
     // Set cmd
     mlxCfgStatus setDevCfg();
     // reset Cmd
     mlxCfgStatus resetDevsCfg();
-    mlxCfgStatus resetDevCfg(const char* dev);
+    mlxCfgStatus resetDevCfg(const char *dev);
     // Set Raw TLV file
     mlxCfgStatus setDevRawCfg();
     mlxCfgStatus backupCfg();
@@ -152,13 +155,14 @@ private:
     mlxCfgStatus createConf();
     mlxCfgStatus apply();
 
-    bool askUser(const char* question);
-    mlxCfgStatus err(bool report, const char* errMsg, ...);
+    bool askUser(const char *question, bool add_prefix=true, bool add_suffix=true);
+    mlxCfgStatus err(bool report, const char *errMsg, ...);
     void printErr();
     // data members
     MlxCfgParams _mlxParams;
     std::string _errStr;
     MlxCfgAllInfo _allInfo;
+    dm_dev_id_t _devType;
 
 };
 
