@@ -52,6 +52,9 @@
 
 using namespace std;
 
+typedef struct reg_access_hca_mqis_reg mqisReg;
+#define MAX_REG_DATA 128
+
 void dealWithSignal()
 {
     int sig;
@@ -68,13 +71,13 @@ void dealWithSignal()
     return;
 }
 
-MError mnvaCom5thGen(mfile* mf, u_int8_t* buff, u_int16_t len, u_int32_t tlvType,
-        reg_access_method_t method, QueryType qT)
+MError mnvaCom5thGen(mfile *mf, u_int8_t *buff, u_int16_t len, u_int32_t tlvType,
+                     reg_access_method_t method, QueryType qT)
 {
     struct tools_open_nvda mnvaTlv;
     memset(&mnvaTlv, 0, sizeof(struct tools_open_nvda));
 
-    if (method == REG_ACCESS_METHOD_GET) {
+    if (method == REG_ACCESS_METHOD_GET && mf->tp != MST_IB) {
         mnvaTlv.nv_hdr.length = sizeof(mnvaTlv.data);
     } else {
         mnvaTlv.nv_hdr.length = len;
@@ -104,8 +107,8 @@ MError mnvaCom5thGen(mfile* mf, u_int8_t* buff, u_int16_t len, u_int32_t tlvType
     return ME_OK;
 }
 
-MError nvqcCom5thGen(mfile* mf, u_int32_t tlvType, bool& suppRead,
-                          bool& suppWrite, u_int32_t& version)
+MError nvqcCom5thGen(mfile *mf, u_int32_t tlvType, bool& suppRead,
+                     bool& suppWrite, u_int32_t& version)
 {
     struct tools_open_nvqc nvqcTlv;
     memset(&nvqcTlv, 0, sizeof(struct tools_open_nvqc));
@@ -127,7 +130,7 @@ MError nvqcCom5thGen(mfile* mf, u_int32_t tlvType, bool& suppRead,
     return ME_OK;
 }
 
-MError nvdiCom5thGen(mfile* mf, u_int32_t tlvType)
+MError nvdiCom5thGen(mfile *mf, u_int32_t tlvType)
 {
     struct tools_open_nvdi nvdiTlv;
     memset(&nvdiTlv, 0, sizeof(struct tools_open_nvdi));
@@ -155,7 +158,7 @@ MError nvdiCom5thGen(mfile* mf, u_int32_t tlvType)
 bool strToNum(string str, u_int32_t& num, int base)
 {
     char *endp;
-    char* numStr = strcpy(new char[str.size() + 1],str.c_str());
+    char *numStr = strcpy(new char[str.size() + 1], str.c_str());
     num = strtoul(numStr, &endp, base);
     if (*endp) {
         delete[] numStr;
@@ -187,8 +190,8 @@ vector<string> splitStr(const string s, char d)
     stringstream ss(s);
     string to;
 
-    while(std::getline(ss, to, d)){
-      v.push_back(to);
+    while (std::getline(ss, to, d)) {
+        v.push_back(to);
     }
 
     return v;
@@ -196,8 +199,8 @@ vector<string> splitStr(const string s, char d)
 
 string mlxcfg_ltrim(string s)
 {
-    const char* cs = s.c_str();
-    while(isspace(*cs)){
+    const char *cs = s.c_str();
+    while (isspace(*cs)) {
         cs++;
     }
     return string(cs);
@@ -207,13 +210,13 @@ string mlxcfg_rtrim(string s)
 {
     //todo rewrite it
     unsigned int i = s.size();
-    if(i == 0) {
+    if (i == 0) {
         return s;
     }
     while (--i > 0 && isspace(s[i])) {
         ;
     }
-    if(i == 0 && isspace(s[i])) {
+    if (i == 0 && isspace(s[i])) {
         return "";
     }
     return s.substr(0, i + 1);
@@ -226,37 +229,51 @@ string mlxcfg_trim(string s)
 
 string writerIdToStr(WriterId writerId)
 {
-    switch(writerId) {
-        case WRITER_ID_UNSPECIFIED:
-            return "Unspecified";
-        case WRITER_ID_CHASSIS_BMC:
-            return "Chassis BMC";
-        case WRITER_ID_MAD:
-            return "MAD";
-        case WRITER_ID_BMC:
-            return "BMC";
-        case WRITER_ID_CMD_IF:
-            return "CMD IF";
-        case WRITER_ID_ICMD:
-            return "ICMD";
-        case WRITER_ID_ICMD_UEFI_HII:
-            return "ICMD UEFI HII";
-        case WRITER_ID_ICMD_UEFI_CLP:
-            return "ICMD UEFI CLP";
-        case WRITER_ID_ICMD_FLEXBOOT:
-            return "ICMD FLEXBOOT";
-        case WRITER_ID_ICMD_MLXCONFIG:
-            return "ICMD MLXCONFIG";
-        case WRITER_ID_ICMD_USER1:
-            return "ICMD USER1";
-        case WRITER_ID_ICMD_USER2:
-            return "ICMD USER2";
-        case WRITER_ID_ICMD_MLXCONFIG_SET_RAW:
-            return "ICMD MLXCONFIG SET RAW";
-        case WRITER_ID_OTHER:
-            return "OTHER";
-        default:
-            return "Unknown";
+    switch (writerId) {
+    case WRITER_ID_UNSPECIFIED:
+        return "Unspecified";
+
+    case WRITER_ID_CHASSIS_BMC:
+        return "Chassis BMC";
+
+    case WRITER_ID_MAD:
+        return "MAD";
+
+    case WRITER_ID_BMC:
+        return "BMC";
+
+    case WRITER_ID_CMD_IF:
+        return "CMD IF";
+
+    case WRITER_ID_ICMD:
+        return "ICMD";
+
+    case WRITER_ID_ICMD_UEFI_HII:
+        return "ICMD UEFI HII";
+
+    case WRITER_ID_ICMD_UEFI_CLP:
+        return "ICMD UEFI CLP";
+
+    case WRITER_ID_ICMD_FLEXBOOT:
+        return "ICMD FLEXBOOT";
+
+    case WRITER_ID_ICMD_MLXCONFIG:
+        return "ICMD MLXCONFIG";
+
+    case WRITER_ID_ICMD_USER1:
+        return "ICMD USER1";
+
+    case WRITER_ID_ICMD_USER2:
+        return "ICMD USER2";
+
+    case WRITER_ID_ICMD_MLXCONFIG_SET_RAW:
+        return "ICMD MLXCONFIG SET RAW";
+
+    case WRITER_ID_OTHER:
+        return "OTHER";
+
+    default:
+        return "Unknown";
     }
 };
 
@@ -275,27 +292,37 @@ void copyBytesVectorToDwVector(const vector<u_int8_t>& bV, vector<u_int32_t>& dw
 
 string parseIndexStr(const string& indexedMlxconfigName)
 {
-    return indexedMlxconfigName.substr(indexedMlxconfigName.find('[') + 1,
-                             indexedMlxconfigName.find(']') - indexedMlxconfigName.find('[') - 1);
+    size_t pl = indexedMlxconfigName.find('[');
+    size_t pr = indexedMlxconfigName.find(']');
+    if (pl == std::string::npos || pr == std::string::npos) {
+        throw MlxcfgException("Expected a parameter with index");
+    }
+    return indexedMlxconfigName.substr(pl + 1,
+                                       pr - pl - 1);
 }
 
 void parseIndexedMlxconfigName(const string& indexedMlxconfigName, string& mlxconfigName, u_int32_t& index)
 {
-   string indexStr = parseIndexStr(indexedMlxconfigName);
+    string indexStr = parseIndexStr(indexedMlxconfigName);
 
     if (!strToNum(indexStr, index)) {
-       throw MlxcfgException("Can not parse the index of %s\n", indexedMlxconfigName.c_str());
+        throw MlxcfgException("Can not parse the index of %s\n", indexedMlxconfigName.c_str());
     }
 
-    mlxconfigName = indexedMlxconfigName.substr(0, indexedMlxconfigName.find('['));
+    size_t p = indexedMlxconfigName.find('[');
+    if (p == std::string::npos) {
+        throw MlxcfgException("Expected a parameter with index");
+    }
+    mlxconfigName = indexedMlxconfigName.substr(0, p);
 }
 
 void extractIndexes(const string& indexesStr, vector<u_int32_t>& indexes)
 {
-    if (indexesStr.find("..") != string::npos) {
+    size_t p = indexesStr.find("..");
+    if (p != string::npos) {
         unsigned int leftIndex = 0, rightIndex = 0;
-        string leftIndexStr = indexesStr.substr(0, indexesStr.find(".."));
-        string rightIndexStr = indexesStr.substr(indexesStr.find("..") + 2);
+        string leftIndexStr = indexesStr.substr(0, p);
+        string rightIndexStr = indexesStr.substr(p + 2);
         if (!strToNum(leftIndexStr, leftIndex)) {
             throw MlxcfgException("Can not parse the index %s", leftIndexStr.c_str());
         }
@@ -304,7 +331,7 @@ void extractIndexes(const string& indexesStr, vector<u_int32_t>& indexes)
         }
         if (leftIndex > rightIndex) {
             throw MlxcfgException("Left index %d can not be greater than right index %d",
-                                   leftIndex, rightIndex);
+                                  leftIndex, rightIndex);
         }
         while (rightIndex >= leftIndex) {
             indexes.push_back(leftIndex);
@@ -317,7 +344,7 @@ void extractIndexes(const string& indexesStr, vector<u_int32_t>& indexes)
         }
         indexes.push_back(index);
     }
-    
+
 }
 
 bool isIndexedMlxconfigName(const string& mlxconfigName)
@@ -325,12 +352,81 @@ bool isIndexedMlxconfigName(const string& mlxconfigName)
     return (mlxconfigName.find("[") != string::npos);
 }
 
-MlxcfgException::MlxcfgException(const char* fmt, ...){
-    char tmp[1024];
+bool getDeviceInformationString(const char* dev, info_type_t op, vector<char>& infoString)
+{
+    mqisReg mqisRegister;
+    mfile* mf;
+    mf = mopen(dev);
+    if (!mf) {
+       return false;
+    }
+    reg_access_status_t rc;
+    int maxDataSize = mget_max_reg_size(mf, MACCESS_REG_METHOD_GET) - sizeof(mqisRegister);
+    if (maxDataSize > MAX_REG_DATA) {
+        maxDataSize = sizeof(mqisRegister);
+    }
+    std::vector<u_int32_t> dataVector(maxDataSize / 4, 0);
+
+    memset(&mqisRegister, 0, sizeof(mqisReg));
+    if (op == Device_Name){
+       mqisRegister.info_type = 0x1;
+    } else if (op == Device_Description) {
+       mqisRegister.info_type = 0x2;
+    } else {
+        mclose(mf);
+        return false;
+    }
+    mqisRegister.read_length = maxDataSize;
+    mqisRegister.info_string = dataVector.data();
+    mft_signal_set_handling(1);
+
+    rc = reg_access_mqis(mf, REG_ACCESS_METHOD_GET, &mqisRegister);
+    dealWithSignal();
+    if (rc) {
+       mclose(mf);
+       return false;
+    }
+    int infoSize = mqisRegister.info_length;
+    if (infoSize == 0) {
+        return false;
+    }
+    infoString.resize(infoSize+1, 0);
+    if (mqisRegister.info_length > maxDataSize) {
+        dataVector.resize((infoSize + 3) / 4);
+        int leftSize = infoSize - maxDataSize;
+        while (leftSize > 0) {
+            mqisRegister.read_offset = infoSize - leftSize;
+            mqisRegister.read_length = leftSize > maxDataSize ? maxDataSize : leftSize;
+            mqisRegister.info_string = dataVector.data() + (mqisRegister.read_offset / 4);
+            mft_signal_set_handling(1);
+
+            rc = reg_access_mqis(mf, REG_ACCESS_METHOD_GET, &mqisRegister);
+            dealWithSignal();
+            if (rc) {
+               mclose(mf);
+               return false;
+            }
+            leftSize = leftSize - maxDataSize;
+        }
+    }
+
+    memcpy(infoString.data(), dataVector.data(), infoSize);
+    mclose(mf);
+    string str = string (infoString.data());
+    if (str.length() == 0) {
+        return false;
+    }
+    return true;
+}
+
+MlxcfgException::MlxcfgException(const char *fmt, ...)
+{
+    const unsigned int max = 1024;
+    char tmp[max];
     va_list args;
 
-    va_start (args, fmt);
-    vsprintf(tmp, fmt, args);
+    va_start(args, fmt);
+    vsnprintf(tmp, max, fmt, args);
     va_end(args);
     _err = tmp;
 }
