@@ -74,7 +74,7 @@
 
 // djgpp stdio does not define vsnprintf. I simply call vsprintf (and pray ...)
         #define vsnprintf(buf, len, format, args) (vsprintf(buf, format, args))
-        #define snprintf(buf, len, format, args...) (sprintf(buf, format, args))
+        #define snprintf(buf, len, format, args ...) (sprintf(buf, format, args))
     #endif // __DJGPP__
 
 #else // __WIN__
@@ -84,13 +84,13 @@
 //
     #include <mtcr.h>
 // Sleep adaptor
-    #define usleep(x) Sleep((x)/1000)
-    #define sleep(x)  Sleep((x)*1000)
+    #define usleep(x) Sleep((x) / 1000)
+    #define sleep(x)  Sleep((x) * 1000)
 
     #define vsnprintf      _vsnprintf
     #define isatty         _isatty
 
-    // MINGW
+// MINGW
     #if defined(__MINGW32__) || (__MINGW64__)
         #define _UNISTD_H // Zlib includes unistd.h which causes some compilation errors.
     #else
@@ -103,95 +103,96 @@
 #include <compatibility.h>
 #include "mlxfwops_com.h"
 
-static inline void be_guid_to_cpu(guid_t* to, guid_t* from) {
-        to->h=__be32_to_cpu(from->h);
-        to->l=__be32_to_cpu(from->l);
+static inline void be_guid_to_cpu(guid_t *to, guid_t *from)
+{
+    to->h = __be32_to_cpu(from->h);
+    to->l = __be32_to_cpu(from->l);
 }
 namespace std {}; using namespace std;
 
 #define TOCPU1(s) s = __be32_to_cpu((u_int32_t)(s));
 #define CPUTO1(s) s = __cpu_to_be32((u_int32_t)(s));
 #define TOCPU(s) do {                                              \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t); ii++,p++) \
-        *p = __be32_to_cpu(*p);                                    \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t); ii++, p++) \
+            *p = __be32_to_cpu(*p);                                    \
+} while (0)
 #define TOCPUn(s, n) do {                                          \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<(n); ii++,p++)                         \
-        *p = __be32_to_cpu(*p);                                    \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < (n); ii++, p++)                         \
+            *p = __be32_to_cpu(*p);                                    \
+} while (0)
 
 #define CPUTOn(s, n) do {                                          \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<(n); ii++,p++)                         \
-        *p = __cpu_to_be32(*p);                                    \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < (n); ii++, p++)                         \
+            *p = __cpu_to_be32(*p);                                    \
+} while (0)
 
 #define TOCPUBY(s) do {                                            \
-    u_int32_t *p = (u_int32_t *)(&s);                              \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t); ii++,p++) \
-        *p = __be32_to_cpu(*p);                                    \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(&s);                              \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t); ii++, p++) \
+            *p = __be32_to_cpu(*p);                                    \
+} while (0)
 #define CPUTOBY(s) do {                                            \
-    u_int32_t *p = (u_int32_t *)(&s);                              \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t); ii++,p++) \
-        *p = __cpu_to_be32(*p);                                    \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(&s);                              \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t); ii++, p++) \
+            *p = __cpu_to_be32(*p);                                    \
+} while (0)
 #define TOCPUBY64(s) do {                                          \
-    guid_t *p = s;                              \
-    for (unsigned ii=0; ii<sizeof(s)/sizeof(guid_t); ii++,p++)     \
-        be_guid_to_cpu(p,p);                                       \
-    } while(0)
+        guid_t *p = s;                              \
+        for (unsigned ii = 0; ii < sizeof(s) / sizeof(guid_t); ii++, p++)     \
+            be_guid_to_cpu(p, p);                                       \
+} while (0)
 #define CRC(c, s) do {                                             \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t); ii++)     \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t); ii++)     \
+            c << *p++;                                                 \
+} while (0)
 #define CRCn(c, s, n) do {                                         \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<(n); ii++)                             \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < (n); ii++)                             \
+            c << *p++;                                                 \
+} while (0)
 #define CRCBY(c, s) do {                                           \
-    u_int32_t *p = (u_int32_t *)(&s);                              \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t); ii++)     \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(&s);                              \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t); ii++)     \
+            c << *p++;                                                 \
+} while (0)
 #define CRC1(c, s) do {                                            \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t) - 1; ii++) \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t) - 1; ii++) \
+            c << *p++;                                                 \
+} while (0)
 #define CRC1n(c, s, n) do {                                        \
-    u_int32_t *p = (u_int32_t *)(s);                               \
-    for (u_int32_t ii=0; ii<(n) - 1; ii++)                         \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(s);                               \
+        for (u_int32_t ii = 0; ii < (n) - 1; ii++)                         \
+            c << *p++;                                                 \
+} while (0)
 #define CRC1BY(c, s) do {                                          \
-    u_int32_t *p = (u_int32_t *)(&s);                              \
-    for (u_int32_t ii=0; ii<sizeof(s)/sizeof(u_int32_t) - 1; ii++) \
-        c << *p++;                                                 \
-    } while(0)
+        u_int32_t *p = (u_int32_t*)(&s);                              \
+        for (u_int32_t ii = 0; ii < sizeof(s) / sizeof(u_int32_t) - 1; ii++) \
+            c << *p++;                                                 \
+} while (0)
 
 
-#define CHECKB2(b,o,n,f,p,fp) do { if (!checkBoot2(b,o,n,f,p,fp)) return false; } while (0)
-#define FS3_CHECKB2(b,o,f,p,i) {u_int32_t n; if (!checkBoot2(b,o,n,f,p,i)) return false;}
+#define CHECKB2(b, o, n, f, p, fp) do { if (!checkBoot2(b, o, n, f, p, fp)) {return false;}} while (0)
+#define FS3_CHECKB2(b, o, f, p, i) {u_int32_t n; if (!checkBoot2(b, o, n, f, p, i)) {return false;}}
 
-#define CHECKGN(b,o,n,p,fp) do { if (!checkGen(b,o,n,p,fp)) return false; } while (0)
-#define CHECKLS(o,s,p) do { if (!checkList(o,s,p)) return false; } while(0)
-#define READ4(f,o,d,p) do { if (!f.read(o,d)) {  \
-    return errmsg("%s - read error (%s)\n", p, f.err()); }} while (0)
-#define READ4_NOERRMSG(f,o,d) do { if (!f.read(o,d)) {  \
-		return false; }} while (0)
-#define READBUF(f,o,d,l,p) do { if (!f.read(o,d,l)) { \
-    return errmsg("%s - read error (%s)\n", p, f.err()); }} while (0)
-#define READALLOCBUF(f,o,d,l,p) do {\
-    d = (u_int8_t*) malloc(sizeof(u_int8_t) * l);\
-    if (!f.read(o,d,l)) { \
-        free(d);\
-        return errmsg("%s - read error (%s)\n", p, f.err());\
-    }\
+#define CHECKGN(b, o, n, p, fp) do { if (!checkGen(b, o, n, p, fp)) {return false;}} while (0)
+#define CHECKLS(o, s, p) do { if (!checkList(o, s, p)) {return false;}} while (0)
+#define READ4(f, o, d, p) do { if (!f.read(o, d)) {  \
+                                   return errmsg("%s - read error (%s)\n", p, f.err()); }} while (0)
+#define READ4_NOERRMSG(f, o, d) do { if (!f.read(o, d)) {  \
+                                         return false; }} while (0)
+#define READBUF(f, o, d, l, p) do { if (!f.read(o, d, l)) { \
+                                        return errmsg("%s - read error (%s)\n", p, f.err()); }} while (0)
+#define READALLOCBUF(f, o, d, l, p) do { \
+        d = (u_int8_t*) malloc(sizeof(u_int8_t) * l); \
+        if (!f.read(o, d, l)) { \
+            free(d); \
+            return errmsg("%s - read error (%s)\n", p, f.err()); \
+        } \
 } while (0)
 
 #define FS2_BOOT_START   0x38
@@ -201,7 +202,7 @@ namespace std {}; using namespace std;
 #define SPECTRUM_HW_ID   585
 #define SWITCH_IB2_HW_ID 587
 #define QUANTUM_HW_ID    589
-#define SPECTRUM2_HW_ID  591
+#define SPECTRUM2_HW_ID  590
 
 
 #define CX4_HW_ID         521
@@ -218,11 +219,11 @@ namespace std {}; using namespace std;
 
 // FS3 defines
 #define FS3_BOOT_START        FS2_BOOT_START
-#define FS3_BOOT_START_IN_DW  FS3_BOOT_START/4
+#define FS3_BOOT_START_IN_DW  FS3_BOOT_START / 4
 
 // FS4 defines
 #define FS4_BOOT_START        0x10
-#define FS4_BOOT_START_IN_DW  FS4_BOOT_START/4
+#define FS4_BOOT_START_IN_DW  FS4_BOOT_START / 4
 #define FS4_BOOT2_START        0x188
 #define FS4_HW_PTR_START       0x18
 #define FS4_HW_PTR_SIZE_DW     CX5FW_HW_POINTERS_SIZE / 4
@@ -309,9 +310,12 @@ typedef enum fs3_section {
     // FS3_DBG_LOG_MAP = 0x30 - in practice its unused and DBG_FW_INI is found in that section instead
     FS3_DBG_FW_PARAMS = 0x32,
     FS3_FW_ADB        = 0x33,
-    FS3_IMAGE_SIGNATURE = 0xa0,
-    FS3_PUBLIC_KEYS   = 0xa1,
+    FS3_IMAGE_SIGNATURE_256 = 0xa0,
+    FS3_PUBLIC_KEYS_2048   = 0xa1,
     FS3_FORBIDDEN_VERSIONS = 0xa2,
+    FS3_IMAGE_SIGNATURE_512 = 0xa3,
+    FS3_PUBLIC_KEYS_4096 = 0xa4,
+    FS3_HMAC          = 0xa5,
     FS3_MFG_INFO      = 0xe0,
     FS3_DEV_INFO      = 0xe1,
     FS3_NV_DATA1      = 0xe2,
@@ -319,6 +323,7 @@ typedef enum fs3_section {
     FS3_NV_DATA2      = 0xe4,
     FS3_FW_NV_LOG     = 0xe5,
     FS3_NV_DATA0      = 0xe6, // replaces FS3_NV_DATA1
+    FS3_CRDUMP_MASK_DATA = 0xe9,
     FS4_HW_PTR        = 0xfb,
     FS4_TOOLS_AREA    = 0xfc,
     FS3_ITOC          = 0xfd,
@@ -362,7 +367,8 @@ enum CommandType {
     CMD_BURN_VPD,
     CMD_SET_SIGNATURE,
     CMD_SET_PUBLIC_KEYS,
-    CMD_SET_FORBIDDEN_VERSIONS
+    CMD_SET_FORBIDDEN_VERSIONS,
+    CMD_SET_HMAC
 };
 
 
@@ -375,25 +381,27 @@ struct GPH {
 
 #define MAX_SECTION_SIZE 0x400000
 
-const u_int32_t BOARD_ID_BSN_LEN=64;
-const u_int32_t BOARD_ID_BID_LEN=32;
-const u_int32_t BOARD_ID_PID=7;
+const u_int32_t BOARD_ID_BSN_LEN = 64;
+const u_int32_t BOARD_ID_BID_LEN = 32;
+const u_int32_t BOARD_ID_PID = 7;
 
 struct BOARD_ID {
-    char      bsn[BOARD_ID_BSN_LEN];
-    char      bid[BOARD_ID_BID_LEN];
+    char bsn[BOARD_ID_BSN_LEN];
+    char bid[BOARD_ID_BID_LEN];
 };
 
 #define PROFILES_LIST_SECT "Profiles List section"
 #define TLV_FORMAT_SECT    "TLVs format section"
 #define TRACER_HASH_SECT   "Tracer Hash section"
-#define numbel(x) (sizeof(x)/sizeof((x)[0]))
+#define REBOOT_REQUIRED_STR "To load new FW run reboot machine."
+#define REBOOT_OR_FWRESET_REQUIRED_STR "To load new FW run mlxfwreset or reboot machine."
+#define numbel(x) (sizeof(x) / sizeof((x)[0]))
 
 void report(const char *format, ...);
 void report_callback(f_prog_func_str func_str, const char *format, ...);
 void report_err(char err_buff[MAX_ERR_STR_LEN], const char *format, ...);
 void report_warn(const char *format, ...);
-void report_repair_msg(const char* common_msg);
+void report_repair_msg(const char *common_msg);
 
 
 
@@ -406,24 +414,25 @@ class MLXFWOP_API FlintErrMsg {
 public:
     FlintErrMsg() : _err(0), _errCode(0){}
     ~FlintErrMsg()                { err_clear();}
-    const char *err() const  { return _err;}
+    const char* err() const { return _err;}
     void err_clear();
     int getErrorCode() {return _errCode;}
 
 protected:
 
-    char *vprint(const char *format, va_list args)
+    char* vprint(const char *format, va_list args)
     {
         const int INIT_VAL = 1024;
-        int       max_str, max_buf = INIT_VAL;
+        int max_str, max_buf = INIT_VAL;
         char      *out_buf;
 
         while (1) {
             out_buf = new char[max_buf];
             max_str = max_buf - 1;
 
-            if (vsnprintf(out_buf, max_str, format, args) < max_str)
+            if (vsnprintf(out_buf, max_str, format, args) < max_str) {
                 return out_buf;
+            }
             delete [] out_buf;
             max_buf *= 2;
         }
@@ -432,33 +441,33 @@ protected:
 
     bool errmsg(const char *format, ...)
 #ifdef __GNUC__
-    __attribute__ ((format (printf, 2, 3)))
+    __attribute__ ((format(printf, 2, 3)))
 #endif
     ;
 
     bool errmsg(int errorCode, const char *format, ...)
 #ifdef __GNUC__
-    __attribute__ ((format (printf, 3, 4)))
+    __attribute__ ((format(printf, 3, 4)))
 #endif
     ;
 
     // this is abit ugly as there are no checks on the normalFmt string
     bool errmsgAdv(bool showAdv, const char *normalFmt, const char *AdvFmt, ...)
 #ifdef __GNUC__
-    __attribute__ ((format (printf, 4, 5)))
+    __attribute__ ((format(printf, 4, 5)))
 #endif
     ;
 
     int errmsgWCode(int errorCode, const char *format, ...)
 #ifdef __GNUC__
-    __attribute__ ((format (printf, 3, 4)))
+    __attribute__ ((format(printf, 3, 4)))
 #endif
     ;
 
 private:
 
     char       *_err;
-    int         _errCode;
+    int _errCode;
 };
 
 
@@ -474,53 +483,54 @@ public:
     Crc16(bool d = false) : _debug(d) { clear();}
     u_int16_t      get()              { return _crc;}
     void           clear()            { _crc = 0xffff;}
-    void           operator<<(u_int32_t val) { add(val);}
+    void operator<<(u_int32_t val) { add(val);}
     void           add(u_int32_t val);
     void           finish();
 private:
-    u_int16_t      _crc;
-    bool           _debug;
+    u_int16_t _crc;
+    bool _debug;
 };
 
 
 class u_int32_ba {
 public:
     u_int32_ba(u_int32_t i = 0) :
-    _bits(i),
-    _rbits(_bits),
-    _sptr1(0),
-    _eptr(31)    {}
+        _bits(i),
+        _rbits(_bits),
+        _sptr1(0),
+        _eptr(31)    {}
 
-    u_int32_ba  operator[](u_int32_t idx) {return range((u_int8_t)idx,(u_int8_t)idx);}
-    u_int32_ba& operator= (u_int32_t i)   {_rbits = ((i << _sptr1) & mask()) | (_rbits & ~mask()); return *this;}
-    u_int32_t*  operator& ()              {return &_bits;}
-    operator    u_int32_t ()              {return((mask() & _rbits) >> _sptr1);}
+    u_int32_ba operator[](u_int32_t idx) {return range((u_int8_t)idx, (u_int8_t)idx);}
+    u_int32_ba& operator=(u_int32_t i)   {_rbits = ((i << _sptr1) & mask()) | (_rbits & ~mask()); return *this;}
+    u_int32_t*  operator&()              {return &_bits;}
+    operator    u_int32_t()              {return((mask() & _rbits) >> _sptr1);}
 
-    u_int32_ba  range     (u_int8_t eptr,
-                           u_int8_t sptr) {return u_int32_ba(*this,eptr,sptr);}
+    u_int32_ba  range(u_int8_t eptr,
+                      u_int8_t sptr) {return u_int32_ba(*this, eptr, sptr);}
 
 private:
     u_int32_ba(u_int32_ba& other, u_int8_t eptr, u_int8_t sptr) :
-    _bits(other._bits),
-    _rbits(other._bits),
-    _sptr1(sptr),
-    _eptr(eptr) {}
+        _bits(other._bits),
+        _rbits(other._bits),
+        _sptr1(sptr),
+        _eptr(eptr) {}
 
-    u_int32_t  mask       () {
+    u_int32_t  mask()
+    {
         u_int32_t s_msk = (u_int32_t)-1; // start mask
         u_int32_t e_msk = (u_int32_t)-1; // end mask
 
         s_msk = (s_msk << _sptr1);
-        e_msk = (_eptr >= (sizeof(_bits)*8-1)) ? e_msk : ~(e_msk << (_eptr+1));
+        e_msk = (_eptr >= (sizeof(_bits) * 8 - 1)) ? e_msk : ~(e_msk << (_eptr + 1));
 
         return(s_msk & e_msk);
     };
 
-    u_int32_t  _bits;
+    u_int32_t _bits;
     u_int32_t& _rbits;
 
-    u_int8_t   _sptr1;
-    u_int8_t   _eptr;
+    u_int8_t _sptr1;
+    u_int8_t _eptr;
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -536,11 +546,11 @@ private:
 class Aligner {
 public:
     Aligner(u_int32_t log2_alignment_size) :
-    _curr_addr(0),
-    _curr_size(0),
-    _log2_alignment_size(log2_alignment_size),
-    _alignment_size(1 << log2_alignment_size),
-    _alignment_mask(_alignment_size - 1)
+        _curr_addr(0),
+        _curr_size(0),
+        _log2_alignment_size(log2_alignment_size),
+        _alignment_size(1 << log2_alignment_size),
+        _alignment_mask(_alignment_size - 1)
     {
         if (_log2_alignment_size == 0) {
             _log2_alignment_size = 31;
@@ -549,12 +559,14 @@ public:
         }
     }
 
-    void Init        (u_int32_t  addr, u_int32_t  size) {
+    void Init(u_int32_t addr, u_int32_t size)
+    {
         _curr_addr = addr;
         _curr_size = size;
     }
 
-    bool GetNextChunk(u_int32_t& chunk_addr, u_int32_t& chunk_size) {
+    bool GetNextChunk(u_int32_t& chunk_addr, u_int32_t& chunk_size)
+    {
         if (_curr_size == 0) {
             return false;
         }
