@@ -2,7 +2,7 @@
  *
  * flint_io.cpp - FLash INTerface
  *
- * Copyright (c) 2013 Mellanox Technologies Ltd.  All rights reserved.
+ * Copyright (c) 2011 Mellanox Technologies Ltd.  All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -31,6 +31,9 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
+ *  Version: $Id: flint_io.cpp 7522 2011-11-16 15:37:21Z mohammad $
+ *
  */
 
 #include <errno.h>
@@ -40,7 +43,15 @@
 extern bool _no_erase;
 extern bool _no_burn;
 
-extern const char* g_sectNames[];
+extern const char *g_sectNames[];
+
+#ifdef UEFI_BUILD
+// no signal handling.
+void mft_signal_set_handling(int isOn)
+{
+    return;
+}
+#endif
 
 ////////////////////////////////////////////////////////////////////////
 //
@@ -52,9 +63,7 @@ bool FImage::open(const char *fname, bool read_only, bool advErr)
 {
 
 #ifndef UEFI_BUILD
-//#ifndef 1
-    int                fsize;
-    int                r_cnt;
+    int fsize;
     FILE              *fh;
 
     (void)read_only;  // FImage can be opened only for read so we ignore compiler warnings
@@ -86,18 +95,9 @@ bool FImage::open(const char *fname, bool read_only, bool advErr)
                       fname);
     }
 
-    _buf = new u_int32_t[fsize/4];
-    if ((r_cnt = fread(_buf, 1, fsize, fh)) != fsize) {
-        fclose(fh);
-        if (r_cnt < 0) {
-            return errmsg("Read error on file \"%s\" - %s",fname, strerror(errno));
-        } else {
-            return errmsg("Read error on file \"%s\" - read only %d bytes (from %ld)",
-                          fname, r_cnt, (unsigned long)fsize);
-        }
-    }
-
+    _fname = fname;
     _len = fsize;
+    _isFile = true;
     fclose(fh);
     return true;
 #else
@@ -107,8 +107,8 @@ bool FImage::open(const char *fname, bool read_only, bool advErr)
 
 bool FImage::open(u_int32_t *buf, u_int32_t len, bool advErr)
 {
-    _buf = new u_int32_t[len / 4];
-    memcpy(_buf, buf, len);
+    _buf.resize(len);
+    memcpy(_buf.data(), buf, len);
     _len = len;
     _advErrors = advErr;
     return true;
@@ -116,10 +116,47 @@ bool FImage::open(u_int32_t *buf, u_int32_t len, bool advErr)
 ////////////////////////////////////////////////////////////////////////
 void FImage::close()
 {
-    delete [] _buf;
-    _buf = 0;
+    _fname = (const char*)NULL;
+    _buf.resize(0);
+    _len = 0;
+    _isFile = false;
 } // FImage::close
 
+/////////////////////////////////////////////////////////////////////////
+u_int32_t* FImage::getBuf()
+{
+    if (_isFile) {
+        // Read the entire file on demand
+        FILE *fh = fopen(_fname, "rb");
+        int r_cnt;
+        u_int32_t *retBuf;
+        if (!fh) {
+            errmsg("Can not open file \"%s\" - %s", _fname, strerror(errno));
+            return (u_int32_t*)NULL;
+        }
+        _buf.resize(_len);
+        if ((r_cnt = fread(_buf.data(), 1, _len, fh)) != (int)_len) {
+            if (r_cnt < 0) {
+                errmsg("Read error on file \"%s\" - %s", _fname, strerror(errno));
+                retBuf = (u_int32_t*)NULL;
+                goto cleanup;
+            } else {
+                errmsg("Read error on file \"%s\" - read only %d bytes (from %ld)",
+                       _fname, r_cnt, (unsigned long)_len);
+                retBuf = (u_int32_t*)NULL;
+                goto cleanup;
+            }
+        }
+        _isFile = false;
+        retBuf = (u_int32_t*)_buf.data();
+cleanup:
+        fclose(fh);
+        return retBuf;
+    } else {
+        return (u_int32_t*)_buf.data();
+    }
+}
+
 ////////////////////////////////////////////////////////////////////////
 bool FImage::read(u_int32_t addr, u_int32_t *data)
 {
@@ -127,16 +164,16 @@ bool FImage::read(u_int32_t addr, u_int32_t *data)
 } // FImage::read
 
 ////////////////////////////////////////////////////////////////////////
-bool FImage::read(u_int32_t addr, void *data, int len, bool, const char*)
+bool FImage::read(u_int32_t addr, void *data, int len, bool verbose, const char *message)
 {
+    (void)verbose;
+    (void) message;
 
-    if (addr & 0x3) {
-        return errmsg("Address should be 4-bytes aligned.");
+    if (!readWriteCommCheck(addr, len)) {
+        return false;
     }
-    if (len & 0x3) {
-        return errmsg("Length should be 4-bytes aligned.");
-    }
-    if (!_buf) {
+
+    if (!_isFile && _buf.size() == 0) {
         return errmsg("read() when not opened");
     }
 
@@ -151,13 +188,28 @@ bool FImage::read(u_int32_t addr, void *data, int len, bool, const char*)
     u_int32_t chunk_addr;
     u_int32_t chunk_size;
     Aligner align(_log2_chunk_size);
-    align.Init (addr, len);
+    align.Init(addr, len);
     while (align.GetNextChunk(chunk_addr, chunk_size)) {
         u_int32_t phys_addr = cont2phys(chunk_addr);
-
-        memcpy((u_int8_t*)data + (chunk_addr - addr),
-               (u_int8_t*)_buf +  phys_addr,
-               chunk_size);
+        if (_isFile) {
+            FILE *fh = fopen(_fname, "rb");
+            if (!fh) {
+                return errmsg("Can not open file \"%s\" - %s", _fname, strerror(errno));
+            }
+            if (fseek(fh, phys_addr, SEEK_SET) != 0) {
+                fclose(fh);
+                return errmsg("Failed to read from FW file, offset: %#x - %s", phys_addr, strerror(errno));
+            }
+            if (fread((u_int8_t*)data + (chunk_addr - addr), chunk_size, 1, fh) != 1) {
+                fclose(fh);
+                return errmsg("Failed to read from FW file, offset: %#x - %s", phys_addr, strerror(errno));
+            }
+            fclose(fh);
+        } else {
+            memcpy((u_int8_t*)data + (chunk_addr - addr),
+                   _buf.data() +  phys_addr,
+                   chunk_size);
+        }
     }
 
     return true;
@@ -189,7 +241,92 @@ u_int32_t FImage::get_sector_size()
     }
 }
 
+bool FImage::readFileGetBuffer(std::vector<u_int8_t>& dataBuf)
+{
+    int fileSize;
+    FILE *fh;
 
+    if (!getFileSize(fileSize)) {
+        return false;
+    }
+    dataBuf.resize(fileSize);
+    if ((fh = fopen(_fname, "rb")) == NULL) {
+        return errmsg("Can not open %s: %s\n", _fname, strerror(errno));
+    }
+
+    if (fread(dataBuf.data(), 1, fileSize, fh) != (size_t)fileSize) {
+        dataBuf.resize(0);
+        fclose(fh);
+        return errmsg("Failed to read entire file %s: %s\n", _fname, strerror(errno));
+    }
+    fclose(fh);
+    return true;
+}
+bool FImage::writeEntireFile(std::vector<u_int8_t>& fileContent)
+{
+    FILE *fh;
+    if ((fh = fopen(_fname, "wb")) == (FILE*)NULL) {
+        return errmsg("Can not open %s: %s\n", _fname, strerror(errno));
+    }
+
+    if (fwrite(fileContent.data(), 1, fileContent.size(), fh) != fileContent.size()) {
+        fclose(fh);
+        return errmsg("Failed to write entire file %s: %s\n", _fname, strerror(errno));
+    }
+    fclose(fh);
+    return true;
+}
+
+bool FImage::getFileSize(int& fileSize)
+{
+    FILE *fh;
+    if ((fh = fopen(_fname, "rb")) == NULL) {
+        return errmsg("Can not open %s: %s\n", _fname, strerror(errno));
+    }
+
+    if (fseek(fh, 0, SEEK_END) < 0) {
+        fclose(fh);
+        return errmsg("Failed to get size of the file \"%s\": %s\n", _fname, strerror(errno));
+    }
+    fileSize = ftell(fh);
+    fclose(fh);
+    if (fileSize < 0) {
+        return errmsg("Failed to get size of the file \"%s\": %s\n", _fname, strerror(errno));
+    }
+    return true;
+}
+
+bool FImage::write(u_int32_t addr, void *data, int cnt)
+{
+    if (!_isFile) {
+        if (_buf.size() < addr + cnt) {
+            _buf.resize(addr + cnt);
+        }
+
+        memcpy(&_buf[addr], data, cnt);
+        return true;
+    }
+
+    if (!readWriteCommCheck(addr, 0)) {
+        return false;
+    }
+    // read entire file
+    std::vector<u_int8_t> dataVec;
+    if (!readFileGetBuffer(dataVec)) {
+        return false;
+    }
+    // modify content (extend if needed)
+    if ((dataVec.size() < addr + cnt) ) {
+        dataVec.resize(addr + cnt);
+    }
+    memcpy(&dataVec[addr], data, cnt);
+    // re-write the file
+    if (!writeEntireFile(dataVec)) {
+        return false;
+    }
+    _len = dataVec.size();
+    return true;
+}
 
 ////////////////////////////////////////////////////////////////////////
 //
@@ -207,11 +344,11 @@ bool Flash::open_com_checks(const char *device, int rc, bool force_lock)
 
     if (rc != MFE_OK) {
         if (rc == MFE_SEM_LOCKED) {
-            return errmsgAdv(_advErrors, "Can not obtain Flash semaphore (62).", "You can run \"flint -clear_semaphore -d <device>\" to force semaphore unlock. See help for details.");
+            return errmsgAdv(_advErrors, "Can not obtain Flash semaphore.", "You can run \"flint -clear_semaphore -d <device>\" to force semaphore unlock. See help for details.");
         }
         if (rc == MFE_LOCKED_CRSPACE) {
             _cr_space_locked = 1;
-            return errmsgAdv(_advErrors, "HW access is disabled on the device.", "\n-E- Run \"flint -d %s hw_access enable\" in order to enable HW access.", device);
+            return errmsgAdv(_advErrors, "HW access is disabled on the device.", "\n-E- Run \"flint -d %s hw_access enable [key]\" in order to enable HW access.", device);
         }
         if (rc == MFE_REG_ACCESS_NOT_SUPPORTED) {
             return errmsgAdv(_advErrors, "The target device FW does not support flash access commands.", "\n-E- Please use the -override_cache_replacement option in order to access the flash directly.");
@@ -220,40 +357,39 @@ bool Flash::open_com_checks(const char *device, int rc, bool force_lock)
             return errmsgAdv(_advErrors, "Flash cache replacement is active.", "\n-E- Please use the -override_cache_replacement option in order to access the flash directly.");
         }
 
-        return errmsg("%s %s", errno == 0 ? "" : strerror(errno), mf_err2str(rc));
+        return errmsg("%s%s%s", errno == 0 ? "" : strerror(errno),  errno == 0 ? "" : ". ", mf_err2str(rc));
     }
 
     rc = mf_get_attr(_mfl, &_attr);
     if (rc != MFE_OK) {
         return errmsg("Failed getting flash attributes for device %s: %s", device,  mf_err2str(rc));
     }
+    _curr_sector_size = _attr.sector_size;
 
-    rc = mf_set_opt(_mfl, MFO_NO_VERIFY, _no_flash_verify? 1: 0);
+    rc = mf_set_opt(_mfl, MFO_NO_VERIFY, _no_flash_verify ? 1 : 0);
     if (rc != MFE_OK) {
         return errmsg("Failed setting no flash verify on device: %s", mf_err2str(rc));
     }
 
 
-    if (_attr.hw_dev_id == 435 || _attr.hw_dev_id == SWITCHX_HW_ID) {
+    if (_attr.hw_dev_id == IS4_HW_ID || _attr.hw_dev_id == SWITCHX_HW_ID) {
         _port_num = 0;
-    } else if (_attr.hw_dev_id == 25204 || _attr.hw_dev_id == 24204) {
-        _port_num = 1;
     } else {
         _port_num = 2;
     }
     return true;
 }
 
-bool Flash::set_no_flash_verify(bool val) {
-
-	_no_flash_verify = val;
-	int rc;
-	if (_mfl) {
-		rc = mf_set_opt(_mfl, MFO_NO_VERIFY, val? 1: 0);
-		if (rc != MFE_OK) {
-			return errmsg("Failed setting no flash verify on device: %s", mf_err2str(rc));
-			}
-	}
+bool Flash::set_no_flash_verify(bool val)
+{
+    int rc;
+    if (_mfl) {
+        rc = mf_set_opt(_mfl, MFO_NO_VERIFY, val ? 1 : 0);
+        if (rc != MFE_OK) {
+            return errmsg("Failed setting no flash verify on device: %s", mf_err2str(rc));
+        }
+    }
+    _no_flash_verify = val;
     return true;
 }
 
@@ -262,24 +398,25 @@ bool Flash::set_no_flash_verify(bool val) {
 
 ////////////////////////////////////////////////////////////////////////
 bool Flash::open(const char *device, bool force_lock, bool read_only, int num_of_banks, flash_params_t *flash_params,
-        int ignore_cashe_replacement, bool advErr)
+                 int ignore_cashe_replacement, bool advErr, int cx3_fw_access)
 {
     // Open device
     int rc;
     _advErrors = advErr;
+    _ignore_cache_replacement = ignore_cashe_replacement ? true : false;
     (void)read_only; // not used , avoid compiler warnings TODO: remove this var from function def
-    rc = mf_open(&_mfl, device, num_of_banks, flash_params, ignore_cashe_replacement);
+    rc = mf_open_adv(&_mfl, device, num_of_banks, flash_params, ignore_cashe_replacement, cx3_fw_access);
     //printf("device: %s , forceLock: %s , read only: %s, num of banks: %d, flash params is null: %s, ocr: %d, rc: %d\n",
     //		device, force_lock? "true":"false", read_only?"true":"false", num_of_banks, flash_params? "no":"yes", ignore_cashe_replacement, rc);
     return open_com_checks(device, rc, force_lock);
 }
 
 ////////////////////////////////////////////////////////////////////////
-bool Flash::open(uefi_Dev_t *uefi_dev, f_fw_cmd fw_cmd_func, bool force_lock, bool advErr)
+bool Flash::open(uefi_Dev_t *uefi_dev, uefi_dev_extra_t *uefi_extra, bool force_lock, bool advErr)
 {
     int rc;
     _advErrors = advErr;
-    rc = mf_open_uefi(&_mfl, uefi_dev, fw_cmd_func);
+    rc = mf_open_uefi(&_mfl, uefi_dev, uefi_extra);
     return open_com_checks("uefi", rc, force_lock);
 }
 
@@ -287,20 +424,22 @@ bool Flash::open(uefi_Dev_t *uefi_dev, f_fw_cmd fw_cmd_func, bool force_lock, bo
 ////////////////////////////////////////////////////////////////////////
 void Flash::close()
 {
-    if (!_mfl)
+    if (!_mfl) {
         return;
+    }
 
     mf_close(_mfl);
     _mfl = 0;
 } // Flash::close
 
 bool Flash::read(u_int32_t addr,
-                 u_int32_t *data) {
+                 u_int32_t *data)
+{
     int rc;
 
     u_int32_t phys_addr = cont2phys(addr);
     // printf("-D- read1: addr = %#x, phys_addr = %#x\n", addr, phys_addr);
-    // here we set a "silent" signal handler and deal with the recieved signal after the read
+    // here we set a "silent" signal handler and deal with the received signal after the read
     mft_signal_set_handling(1);
     rc = mf_read(_mfl, phys_addr, 4, (u_int8_t*)data);
     deal_with_signal();
@@ -315,16 +454,11 @@ bool Flash::read(u_int32_t addr,
 }
 
 ////////////////////////////////////////////////////////////////////////
-bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char* message)
+bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char *message)
 {
     int rc;
-    u_int32_t  perc = 0xffffffff;
-
-    if (addr & 0x3) {
-        return errmsg("Address should be 4-bytes aligned.");
-    }
-    if (len & 0x3) {
-        return errmsg("Length should be 4-bytes aligned.");
+    if (!readWriteCommCheck(addr, len)) {
+        return false;
     }
 
     // Much better perf for read in a single chunk. need to work on progress report though.
@@ -335,7 +469,7 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char*
         u_int32_t chunk_size;
 
         Aligner align(_log2_chunk_size);
-        align.Init (addr, len);
+        align.Init(addr, len);
         while (align.GetNextChunk(chunk_addr, chunk_size)) {
             u_int32_t phys_addr = cont2phys(chunk_addr);
             // printf("-D- write: addr = %#x, phys_addr = %#x\n", chunk_addr, phys_addr);
@@ -351,10 +485,12 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char*
         }
 
     } else {
-        u_int32_t *p = (u_int32_t *)data;
-        for (int i=0; i<len/4; i++) {
-            if (!read(addr, p++))
+        u_int32_t perc = 0xffffffff;
+        u_int32_t *p = (u_int32_t*)data;
+        for (int i = 0; i < len / 4; i++) {
+            if (!read(addr, p++)) {
                 return false;
+            }
 
             addr += 4;
 
@@ -370,6 +506,7 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char*
             }
         }
     }
+
     // Report
     if (verbose) {
         printf("\r%s%%100", message);
@@ -378,26 +515,26 @@ bool Flash::read(u_int32_t addr, void *data, int len, bool verbose, const char*
 
     return true;
 } // Flash::read
-#define DISABLE_CONVERTOR(log2_chunk_size_bak, is_image_in_odd_chunks_bak) {\
-    log2_chunk_size_bak = _log2_chunk_size;\
-    is_image_in_odd_chunks_bak = _is_image_in_odd_chunks;\
-    set_address_convertor(0, 0);\
+#define DISABLE_CONVERTOR(log2_chunk_size_bak, is_image_in_odd_chunks_bak) { \
+        log2_chunk_size_bak = _log2_chunk_size; \
+        is_image_in_odd_chunks_bak = _is_image_in_odd_chunks; \
+        set_address_convertor(0, 0); \
 }
 
-#define NATIVE_PHY_ADDR_FUNC(func, arg) {\
-	bool ret = true;\
-	u_int32_t log2_chunk_size_bak, is_image_in_odd_chunks_bak;\
-	DISABLE_CONVERTOR(log2_chunk_size_bak, is_image_in_odd_chunks_bak);\
-	ret = func arg;\
-	set_address_convertor(log2_chunk_size_bak, is_image_in_odd_chunks_bak);\
-	return ret;\
+#define NATIVE_PHY_ADDR_FUNC(func, arg) { \
+        bool ret = true; \
+        u_int32_t log2_chunk_size_bak, is_image_in_odd_chunks_bak; \
+        DISABLE_CONVERTOR(log2_chunk_size_bak, is_image_in_odd_chunks_bak); \
+        ret = func arg; \
+        set_address_convertor(log2_chunk_size_bak, is_image_in_odd_chunks_bak); \
+        return ret; \
 }
-bool FBase::read_phy (u_int32_t phy_addr, u_int32_t* data)
+bool FBase::read_phy(u_int32_t phy_addr, u_int32_t *data)
 {
     NATIVE_PHY_ADDR_FUNC(read, (phy_addr, data));
 }
 
-bool FBase::read_phy (u_int32_t phy_addr, void* data, int len)
+bool FBase::read_phy(u_int32_t phy_addr, void *data, int len)
 {
     NATIVE_PHY_ADDR_FUNC(read, (phy_addr, data, len));
 }
@@ -407,26 +544,39 @@ bool Flash::write_phy(u_int32_t phy_addr, u_int32_t data)
     NATIVE_PHY_ADDR_FUNC(write, (phy_addr, data));
 }
 
-bool Flash::write_phy(u_int32_t phy_addr, void* data, int cnt, bool noerase)
+bool Flash::write_phy(u_int32_t phy_addr, void *data, int cnt, bool noerase)
+{
+    // Avoid warning
+    (void)noerase;
+    NATIVE_PHY_ADDR_FUNC(write, (phy_addr, data, cnt));
+}
+
+bool Flash::read_modify_write_phy(u_int32_t phy_addr, void *data, int cnt, bool noerase)
 {
     // Avoid warning
     (void)noerase;
     NATIVE_PHY_ADDR_FUNC(write_with_erase, (phy_addr, data, cnt));
 }
 
-bool Flash::erase_sector_phy  (u_int32_t phy_addr)
+bool Flash::read_modify_write(u_int32_t phy_addr, void *data, int cnt, bool noerase)
+{
+    // Avoid warning
+    (void)noerase;
+    return write_with_erase(phy_addr, data, cnt);
+}
+
+bool Flash::erase_sector_phy(u_int32_t phy_addr)
 {
     NATIVE_PHY_ADDR_FUNC(erase_sector, (phy_addr));
 }
 
 
 ////////////////////////////////////////////////////////////////////////
-bool Flash::write  (u_int32_t addr,
-                    void*     data,
-                    int       cnt,
-                    bool      noerase)
+bool Flash::write(u_int32_t addr,
+                  void *data,
+                  int cnt,
+                  bool noerase)
 {
-
     // FIX:
     noerase = _no_erase || noerase;
 
@@ -440,26 +590,26 @@ bool Flash::write  (u_int32_t addr,
 
     if (cont2phys(addr + cnt) > get_size()) {
         return errmsg(
-                     "Trying to write %d bytes to address 0x%x, which exceeds max image size (0x%x - half of total flash size).",
-                     cnt,
-                     addr,
-                     get_size() / 2);
+            "Trying to write %d bytes to address 0x%x, which exceeds max image size (0x%x - half of total flash size).",
+            cnt,
+            addr,
+            get_size() / 2);
     }
 
-    u_int8_t         *p = (u_int8_t *)data;
-    u_int32_t sect_size = get_sector_size();
+    u_int8_t         *p = (u_int8_t*)data;
+    u_int32_t sect_size = get_current_sector_size();
 
     u_int32_t chunk_addr;
     u_int32_t chunk_size;
 
     u_int32_t first_set;
-    for (first_set = 0; ((sect_size >> first_set) & 1) == 0; first_set++ )
+    for (first_set = 0; ((sect_size >> first_set) & 1) == 0; first_set++)
         ;
 
     Aligner align(first_set);
-    align.Init (addr, cnt);
+    align.Init(addr, cnt);
     while (align.GetNextChunk(chunk_addr, chunk_size)) {
-        // Write / Erase in sector_size alligned chunks
+        // Write / Erase in sector_size aligned chunks
         int rc;
 
         if (!noerase) {
@@ -473,8 +623,9 @@ bool Flash::write  (u_int32_t addr,
             }
         }
 
-        if (_no_burn)
+        if (_no_burn) {
             continue;
+        }
 
         // Actual write:
         u_int32_t phys_addr = cont2phys(chunk_addr);
@@ -486,19 +637,19 @@ bool Flash::write  (u_int32_t addr,
         if (rc != MFE_OK) {
             if (rc == MFE_ICMD_BAD_PARAM || rc == MFE_REG_ACCESS_BAD_PARAM) {
                 return errmsg("Flash write of %d bytes to address %s0x%x failed: %s\n"
-                          "    This may indicate that a FW image was already updated on flash, but not loaded by the device.\n"
-                          "    Please load FW on the device (reset device or reboot machine) before burning a new FW.",
-                          chunk_size,
-                          _log2_chunk_size ? "physical " : "",
-                          chunk_addr,
-                          mf_err2str(rc));
+                              "    This may indicate that a FW image was already updated on flash, but not loaded by the device.\n"
+                              "    Please load FW on the device (reset device or reboot machine) before burning a new FW.",
+                              chunk_size,
+                              _log2_chunk_size ? "physical " : "",
+                              chunk_addr,
+                              mf_err2str(rc));
 
             } else {
                 return errmsg("Flash write of %d bytes to address %s0x%x failed: %s",
-                          chunk_size,
-                          _log2_chunk_size ? "physical " : "",
-                          chunk_addr,
-                          mf_err2str(rc));
+                              chunk_size,
+                              _log2_chunk_size ? "physical " : "",
+                              chunk_addr,
+                              mf_err2str(rc));
             }
         }
 
@@ -532,8 +683,7 @@ bool Flash::write(u_int32_t addr, u_int32_t data)
 
 bool Flash::write_sector_with_erase(u_int32_t addr, void *data, int cnt)
 {
-
-    u_int32_t sector_size = _attr.sector_size;
+    u_int32_t sector_size = get_current_sector_size();
     u_int32_t sector_mask = ~(sector_size - 1);
 
     u_int32_t sector = addr & sector_mask;
@@ -544,38 +694,56 @@ bool Flash::write_sector_with_erase(u_int32_t addr, void *data, int cnt)
         return errmsg("data exceeds current sector");
     }
 
-    vector<u_int32_t> buff(sector_size/sizeof(u_int32_t));
-    if (!read(sector, &buff[0] , sector_size)) {
+    vector<u_int32_t> buff(sector_size / sizeof(u_int32_t));
+    if (!read(sector, &buff[0], sector_size)) {
+        return false;
+    }
+    if (!erase_sector(sector)) {
         return false;
     }
 
     memcpy(&buff[word_in_sector], data, cnt);
-    return write(sector, &buff[0], sector_size);
+
+    // no need to erase twice noerase=true
+    return write(sector, &buff[0], sector_size, true);
 }
 
 bool Flash::write_with_erase(u_int32_t addr, void *data, int cnt)
 {
     u_int32_t towrite = (u_int32_t)cnt;
     u_int32_t currSize;
+    u_int32_t currAddr = addr;
+    u_int32_t alreadyWritten = 0;
+    u_int32_t sizeUntillEndOfSector = 0;
+    u_int32_t sector_size = get_current_sector_size();
     while (towrite > 0) {
-        currSize = towrite > _attr.sector_size ? (_attr.sector_size) : towrite;
-        if (!write_sector_with_erase(addr, data, currSize)) {
+        sizeUntillEndOfSector = sector_size - (currAddr & (sector_size - 1));
+        currSize = towrite >  sizeUntillEndOfSector ? sizeUntillEndOfSector : towrite;
+        if (!write_sector_with_erase(currAddr, ((u_int8_t*)data + alreadyWritten), currSize)) {
             return false;
         }
         towrite -= currSize;
+        currAddr += currSize;
+        alreadyWritten += currSize;
     }
     return true;
 }
 
-bool Flash::erase_sector  (u_int32_t addr) {
+bool Flash::erase_sector(u_int32_t addr)
+{
     int rc;
-
     u_int32_t phys_addr = cont2phys(addr);
     mft_signal_set_handling(1);
-    rc = mf_erase_sector(_mfl, phys_addr);
+    if (_flash_working_mode == Flash::Fwm_4KB) {
+        rc = mf_erase_4k_sector(_mfl, phys_addr);
+    } else if (_flash_working_mode == Flash::Fwm_64KB) {
+        rc = mf_erase_64k_sector(_mfl, phys_addr);
+    } else {
+        rc = mf_erase(_mfl, phys_addr);
+    }
     deal_with_signal();
     if (rc != MFE_OK) {
-        if (rc == MFE_REG_ACCESS_RES_NOT_AVLBL) {
+        if (rc == MFE_REG_ACCESS_RES_NOT_AVLBL || rc == MFE_REG_ACCESS_BAD_PARAM) {
             return errmsg("Flash erase of address 0x%x failed: %s\n"
                           "    This may indicate that a FW image was already updated on flash, but not loaded by the device.\n"
                           "    Please load FW on the device (reset device or restart driver) before burning a new FW.",
@@ -602,6 +770,12 @@ bool Flash::enable_hw_access(u_int64_t key)
     return true;
 }
 
+
+bool Flash::is_fifth_gen()
+{
+    return mf_is_fifth_gen(_mfl);
+}
+
 bool Flash::disable_hw_access(void)
 {
     int rc;
@@ -614,25 +788,39 @@ bool Flash::disable_hw_access(void)
 
 }
 
-bool Flash::sw_reset() {
-    if (_attr.hw_dev_id != 435 && _attr.hw_dev_id != SWITCHX_HW_ID) {
-        return errmsg("operation supported only for InfiniScale4 switch and SwitchX over IB interface");
+bool Flash::disable_hw_access(u_int64_t key)
+{
+    int rc;
+    rc = mf_disable_hw_access_with_key(_mfl, key);
+
+    if (rc != MFE_OK) {
+        return errmsg("Disable HW access failed: %s", mf_err2str(rc));
     }
+    return true;
+
+}
+
+bool Flash::sw_reset()
+{
     int rc = mf_sw_reset(_mfl);
     if (rc != MFE_OK) {
+        if (rc == MFE_UNSUPPORTED_DEVICE) {
+            return errmsg("operation supported only for InfiniScale4 switch, SwitchX and SwitchIB over IB interface");
+        }
         return errmsg("%s (%s)", errno == 0 ? "" : strerror(errno), mf_err2str(rc));
     }
     return true;
 }
 
 
-bool Flash::get_attr(ext_flash_attr_t& attr) {
+bool Flash::get_attr(ext_flash_attr_t& attr)
+{
     attr.banks_num = _attr.banks_num;
     attr.hw_dev_id = _attr.hw_dev_id;
     attr.rev_id = _attr.rev_id;
     if (_attr.type_str != NULL) {
         // we don't print the flash type in old devices
-        attr.type_str = strcpy(new char[strlen(_attr.type_str)+1], _attr.type_str);
+        attr.type_str = strcpy(new char[strlen(_attr.type_str) + 1], _attr.type_str);
     }
     attr.size = _attr.size;
     attr.sector_size = _attr.sector_size;
@@ -646,7 +834,7 @@ bool Flash::get_attr(ext_flash_attr_t& attr) {
     }
     // Dummy Cycles query
     if (_attr.dummy_cycles_support) {
-    	attr.mf_get_dummy_cycles_rc = (MfError)mf_get_dummy_cycles(_mfl, &attr.dummy_cycles);
+        attr.mf_get_dummy_cycles_rc = (MfError)mf_get_dummy_cycles(_mfl, &attr.dummy_cycles);
     }
 
     // Flash write protected info query
@@ -660,48 +848,49 @@ bool Flash::get_attr(ext_flash_attr_t& attr) {
     return true;
 }
 
-const char* Flash::getFlashType() {
+const char* Flash::getFlashType()
+{
     if (_attr.type_str != NULL) {
-    	return _attr.type_str;
+        return _attr.type_str;
     }
     return (const char*)NULL;
 }
 
-#define GET_IN_PARAM(param_in, out, first_op, second_op) {\
-    if (!strcmp(param_in, first_op)) {\
-        out = 1;\
-    } else if (!strcmp(param_in, second_op)) {\
-        out = 0;\
-    } else {\
-        return errmsg("bad argument (%s) it can be "first_op" or " second_op"", param_in);\
-    }\
+#define GET_IN_PARAM(param_in, out, first_op, second_op) { \
+        if (!strcmp(param_in, first_op)) { \
+            out = 1; \
+        } else if (!strcmp(param_in, second_op)) { \
+            out = 0; \
+        } else { \
+            return errmsg("bad argument (%s) it can be " first_op " or " second_op "", param_in); \
+        } \
 }
-bool  Flash::set_attr(char *param_name, char *param_val_str)
+bool Flash::set_attr(char *param_name, char *param_val_str)
 {
     int rc;
     //TODO: make generic function that sets params
     if (!strcmp(param_name, QUAD_EN_PARAM)) {
-        char* endp;
+        char *endp;
         u_int8_t quad_en_val;
         quad_en_val = strtoul(param_val_str, &endp, 0);
         if (*endp != '\0' || quad_en_val > 1) {
-            return errmsg("Bad "QUAD_EN_PARAM" value (%s), it can be 0 or 1\n", param_val_str);
+            return errmsg("Bad " QUAD_EN_PARAM " value (%s), it can be 0 or 1\n", param_val_str);
         }
         rc = mf_set_quad_en(_mfl, quad_en_val);
         if (rc != MFE_OK) {
-            return errmsg("Setting "QUAD_EN_PARAM" failed: (%s)", mf_err2str(rc));
+            return errmsg("Setting " QUAD_EN_PARAM " failed: (%s)", mf_err2str(rc));
         }
     } else if (!strcmp(param_name, DUMMY_CYCLES_PARAM)) {
-        char* endp;
+        char *endp;
         u_int8_t dummy_cycles_val;
         dummy_cycles_val = strtoul(param_val_str, &endp, 0);
         if (*endp != '\0' || dummy_cycles_val < 1 || dummy_cycles_val > 15) {
-        	// value is actually [0.15] but val=0 and val=15 indicate default state (thus they are the same so no need for both values to be accepted)
-            return errmsg("Bad "DUMMY_CYCLES_PARAM" value (%s), it can be [1..15]\n", param_val_str);
+            // value is actually [0.15] but val=0 and val=15 indicate default state (thus they are the same so no need for both values to be accepted)
+            return errmsg("Bad " DUMMY_CYCLES_PARAM " value (%s), it can be [1..15]\n", param_val_str);
         }
         rc = mf_set_dummy_cycles(_mfl, dummy_cycles_val);
         if (rc != MFE_OK) {
-            return errmsg("Setting "DUMMY_CYCLES_PARAM" failed: (%s)", mf_err2str(rc));
+            return errmsg("Setting " DUMMY_CYCLES_PARAM " failed: (%s)", mf_err2str(rc));
         }
     } else if (strstr(param_name, FLASH_NAME) == param_name) {
         char *flash_param, *param_str, *endp, *bank_num_str;
@@ -716,7 +905,6 @@ bool  Flash::set_attr(char *param_name, char *param_val_str)
         if (!strcmp(param_str, WRITE_PROTECT)) {
             write_protect_info_t protect_info;
             char *tb, *num_str, *sec;
-
             if (!strcmp(param_val_str, WP_DISABLED_STR)) {
                 memset(&protect_info, 0, sizeof(protect_info));
             } else {
@@ -724,7 +912,7 @@ bool  Flash::set_attr(char *param_name, char *param_val_str)
                 num_str = strtok((char*)NULL, "-");
                 sec = strtok((char*)NULL, "");
                 if (tb == NULL || num_str == NULL || sec == NULL) {
-                    return errmsg("missing parameters for setting the "WRITE_PROTECT" attribute, see help for more info.");
+                    return errmsg("missing parameters for setting the " WRITE_PROTECT " attribute, see help for more info.");
                 }
                 GET_IN_PARAM(tb, protect_info.is_bottom, WP_BOTTOM_STR, WP_TOP_STR);
                 GET_IN_PARAM(sec, protect_info.is_subsector, WP_SUBSEC_STR, WP_SEC_STR);
@@ -733,10 +921,13 @@ bool  Flash::set_attr(char *param_name, char *param_val_str)
                 if (*endp != '\0') {
                     return errmsg("bad argument (%s), only integer value is allowed.", num_str);
                 }
-           }
+                if (!protect_info.sectors_num) {
+                    return errmsg("Invalid sectors number, Use \"Disabled\" instead.");
+                }
+            }
             rc = mf_set_write_protect(_mfl, bank_num, &protect_info);
             if (rc != MFE_OK) {
-                return errmsg("Setting "WRITE_PROTECT" failed: (%s)", mf_err2str(rc));
+                return errmsg("Setting " WRITE_PROTECT " failed: (%s)", mf_err2str(rc));
             }
         } else {
             return errmsg("Unknown attribute %s.%s", flash_param, param_str);
@@ -746,16 +937,38 @@ bool  Flash::set_attr(char *param_name, char *param_val_str)
     } else {
         return errmsg("Unknown attribute %s", param_name);
     }
-
     return true;
 }
 
+bool Flash::is_flash_write_protected()
+{
+    int bank;
+    int rc;
+    write_protect_info_t protect_info;
+
+    memset(&protect_info, 0x0, sizeof(protect_info));
+    if (_attr.write_protect_support) {
+        for (bank = 0; bank < _attr.banks_num; bank++) {
+            rc = mf_get_write_protect(_mfl, bank, &protect_info);
+            if (rc == MFE_OK && protect_info.sectors_num != 0) {
+                return true;
+            } else {
+                if (rc && rc != MFE_NOT_SUPPORTED_OPERATION) { // We ignore when operation is not supported
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
 void Flash::deal_with_signal()
 {
+#ifndef UEFI_BUILD
     int sig;
     sig = mft_signal_is_fired();
     if (sig) {
-        // reset recieved signal
+        // reset received signal
         mft_signal_set_fired(0);
         // retore prev handler
         mft_signal_set_handling(0);
@@ -763,7 +976,43 @@ void Flash::deal_with_signal()
         raise(sig);
     }
     mft_signal_set_handling(0);
+#endif
     return;
 }
 
+#define FLINT_ERASE_SIZE_HOOK "FLINT_ERASE_SIZE"
+bool Flash::set_flash_working_mode(int mode)
+{
+    if (!_attr.support_sub_and_sector && mode != Flash::Fwm_Default) {
+        return errmsg("Changing Flash IO working mode not supported.");
+    }
+    // Verification Hook
+    if (_attr.support_sub_and_sector) {
+        char *flint_io_env = getenv(FLINT_ERASE_SIZE_HOOK);
+        if (flint_io_env) {
+            int num =  strtoul(flint_io_env, (char**)NULL, 0);
+            if (num == 0x1000 || num == 0x10000) {
+                _curr_sector_size = (u_int32_t)num;
+                _flash_working_mode = _curr_sector_size == 0x1000 ? Flash::Fwm_4KB : Flash::Fwm_64KB;
+                _curr_sector = _curr_sector & ~(_curr_sector_size - 1);
+                return true;
+            }
+        }
+    }
+    // Verification Hook end
 
+    if (mode == Flash::Fwm_Default) {
+        _curr_sector_size = _attr.sector_size;
+        _flash_working_mode = Flash::Fwm_Default;
+    } else if (mode == Flash::Fwm_4KB) {
+        _curr_sector_size = 0x1000;
+        _flash_working_mode = Flash::Fwm_4KB;
+    } else if (mode == Flash::Fwm_64KB) {
+        _curr_sector_size = 0x10000;
+        _flash_working_mode = Flash::Fwm_64KB;
+    } else {
+        return errmsg("Unknown Flash IO working mode: 0x%x", mode);
+    }
+    _curr_sector = _curr_sector & ~(_curr_sector_size - 1);
+    return true;
+}
