@@ -14,12 +14,12 @@
  *      - Redistributions of source code must retain the above
  *        copyright notice, this list of conditions and the following
  *        disclaimer.
- * 
+ *
  *      - Redistributions in binary form must reproduce the above
  *        copyright notice, this list of conditions and the following
  *        disclaimer in the documentation and/or other materials
  *        provided with the distribution.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
@@ -28,6 +28,7 @@
  * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
  */
 
 #include <crdump.h>
@@ -49,22 +50,24 @@
 
 
 // string explaining the cmd-line structure
-char correct_cmdline[] = "   Mellanox "MSTDUMP_NAME" utility, dumps device internal configuration data\n\
-   Usage: "MSTDUMP_NAME" [-full] <device> [i2c-slave] [-v[ersion] [-h[elp]]]\n\n\
+char correct_cmdline[] = "   Mellanox "MSTDUMP_NAME " utility, dumps device internal configuration data\n\
+   Usage: "MSTDUMP_NAME " [-full] <device> [i2c-slave] [-v[ersion] [-h[elp]]]\n\n\
    -full              :  Dump more expanded list of addresses\n\
                          Note : be careful when using this flag, None safe addresses might be read.\n\
-   -v                 :  Display version info\n\
-   -h                 :  Print this help message\n\
+   -v | --version     :  Display version info\n\
+   -h | --help        :  Print this help message\n\
    Example :\n\
-            "MSTDUMP_NAME" "DEV_EXAMPLE"\n";
+            "MSTDUMP_NAME " "DEV_EXAMPLE "\n";
 
 
-void print_dword(crd_dword_t *dword) {
+void print_dword(crd_dword_t *dword)
+{
     printf("0x%8.8x 0x%8.8x\n", dword->addr, dword->data);
 }
 
-int main(int argc, char* argv[]) {
-    int i ;
+int main(int argc, char *argv[])
+{
+    int i;
     mfile *mf;
     int rc;
     int full = 0;
@@ -74,7 +77,7 @@ int main(int argc, char* argv[]) {
     char *endptr;
     u_int8_t new_i2c_slave = 0;
     char device[MAX_DEV_LEN] = {0};
-#if defined(linux)
+#if defined(__linux__) || defined(__FreeBSD__)
     if (geteuid() != 0) {
         printf("-E- Permission denied: User is not root\n");
         return 1;
@@ -88,17 +91,16 @@ int main(int argc, char* argv[]) {
 
     for (i = 1; i < argc; ++i) {
         /* check position-independent flags */
-        if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "-help")) {
+        if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "-help") || !strcmp(argv[i], "--help")) {
             fprintf(stdout, "%s", correct_cmdline);
-            exit (0);
-        }
-        else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "-version")) {
+            exit(0);
+        } else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "-version") || !strcmp(argv[i], "--version"))   {
             print_version_string("mstdump", "");
             exit(0);
-        }
-        else if (!strncmp(argv[i], CAUSE_FLAG, strlen(CAUSE_FLAG))) {
-            if (sscanf(argv[i], CAUSE_FLAG"=%i.%d", &cause_addr, &cause_off) != 2) {
+        } else if (!strncmp(argv[i], CAUSE_FLAG, strlen(CAUSE_FLAG)))   {
+            if (sscanf(argv[i], CAUSE_FLAG "=%i.%d", &cause_addr, &cause_off) != 2) {
                 fprintf(stderr, "Invalid parameters to " CAUSE_FLAG " flag\n");
+                fprintf(stdout, "%s", correct_cmdline);
                 exit(1);
             }
             if (cause_addr < 0 || cause_off < 0) {
@@ -116,21 +118,16 @@ int main(int argc, char* argv[]) {
     }
     if (i >= argc) {
         fprintf(stderr, "Device is not specified in command line. Exiting.\n");
+        fprintf(stdout, "%s", correct_cmdline);
         return 1;
     }
-    strncpy(device, argv[i], MAX_DEV_LEN -1);
-    if (!( mf = mopen_adv((const char *)device, (MType)(MST_DEFAULT | MST_CABLE)))) {
+    strncpy(device, argv[i], MAX_DEV_LEN - 1);
+    if (!( mf = mopen_adv((const char*)device, (MType)(MST_DEFAULT | MST_CABLE)))) {
         fprintf(stderr, "Unable to open device %s. Exiting.\n", argv[i]);
         return 1;
     }
     ++i;    // move past the device parameter
 
-#ifndef MST_UL
-    if (mf->tp == MST_MLNXOS) {
-        mset_cr_access(mf, 1);
-    }
-#endif
-
     if (i < argc && !strncmp(argv[i], CAUSE_FLAG, strlen(CAUSE_FLAG))) {
         i++;
     }
@@ -153,7 +150,7 @@ int main(int argc, char* argv[]) {
         }
     }
     rc = CRD_OK;
-    rc = crd_init(&context, mf, full, cause_addr, cause_off);
+    rc = crd_init(&context, mf, full, cause_addr, cause_off, NULL);
     if (rc) {
         mclose(mf);
         goto error;
@@ -178,8 +175,8 @@ int main(int argc, char* argv[]) {
     mclose(mf);
     return 0;
 
-    error :
-        printf("-E- %s\n", crd_err_str(rc));
-        return rc;
+error:
+    printf("-E- %s\n", crd_err_str(rc));
+    return rc;
 }
 
